CREATE TABLE sscSetting (
	settingID number(10,0) not null, 
	class varchar2(255) default 'com.softslate.commerce.businessobjects.core.SettingBean', 
	code varchar2(100) not null unique, 
	name varchar2(255), 
	description clob, 
	type varchar2(100), 
	valueType varchar2(16) not null, 
	integerValue number(10,0), 
	smallValue varchar2(64), 
	mediumValue varchar2(255), 
	textValue clob, 
	primary key (settingID)
);

CREATE SEQUENCE hibernate_sequence;

CREATE OR REPLACE TRIGGER settingSequenceTrigger 
	BEFORE INSERT ON sscSetting 
	FOR EACH ROW
	WHEN (new.settingID IS NULL)
		BEGIN
			SELECT hibernate_sequence.NEXTVAL
			INTO   :new.settingID
			FROM   dual; 
		END;;
		
CREATE TABLE sscRequestLog (
  requestLogID number(10,0) not null,
  orderID number(10,0),
  orderNumber number(10,0),
  customerID number(10,0),
  customerUserName varchar2(100),
  administratorID number(10,0),
  administratorUserName varchar2(100),
  sessionID varchar2(100),
  ipAddress varchar2(255),
  userAgent varchar2(255),
  servletPath varchar2(255),
  requestURL varchar2(255),
  requestParameters clob,
  referrer varchar2(255),
  isAdminRequest number(1,0) default 0, 
  created varchar2(100),
  searchString varchar2(255),
  class varchar2(255) DEFAULT 'com.softslate.commerce.businessobjects.core.RequestLogBean',
  primary key (requestLogID)
);

CREATE INDEX sscRequestLog_orderID ON sscRequestLog (orderID);
CREATE INDEX sscRequestLog_orderNumber ON sscRequestLog (orderNumber);
CREATE INDEX sscRequestLog_customerID ON sscRequestLog (customerID);
CREATE INDEX sscRequestLog_customerUserName ON sscRequestLog (customerUserName);
CREATE INDEX sscRequestLog_administratorID ON sscRequestLog (administratorID);
CREATE INDEX sscRequestLog_adminUserName ON sscRequestLog (administratorUserName);
CREATE INDEX sscRequestLog_sessionID ON sscRequestLog (sessionID);
CREATE INDEX sscRequestLog_ipAddress ON sscRequestLog (ipAddress);
CREATE INDEX sscRequestLog_userAgent ON sscRequestLog (userAgent);
CREATE INDEX sscRequestLog_servletPath ON sscRequestLog (servletPath);
CREATE INDEX sscRequestLog_requestURL ON sscRequestLog (requestURL);
CREATE INDEX sscRequestLog_created ON sscRequestLog (created);
CREATE INDEX sscRL_searchString ON sscRequestLog (searchString);
