CREATE TABLE sscCategory (
  categoryID INT IDENTITY NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  keywords VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive BIT NOT NULL DEFAULT '1',
  isContentPage BIT NOT NULL DEFAULT '0',
  categoryTreeID INT NOT NULL DEFAULT '1',
  header TEXT,
  footer TEXT,
  smallImage VARCHAR(255) DEFAULT NULL,
  mediumImage VARCHAR(255) DEFAULT NULL,
  largeImage VARCHAR(255) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  parentCategory INT DEFAULT NULL,
  categoryOrder INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.CategoryBean',
  CONSTRAINT PK_categoryID PRIMARY KEY CLUSTERED 
  ( categoryID ) ON [PRIMARY],
  CONSTRAINT AK_categoryCode UNIQUE NONCLUSTERED 
  ( code ) ON [PRIMARY]
);

CREATE INDEX IDX_sscCategory_name ON sscCategory (name);
CREATE INDEX IDX_sscCategory_parentCategory ON sscCategory (parentCategory);
CREATE INDEX IDX_sscCategory_categoryOrder ON sscCategory (categoryOrder);

CREATE TABLE sscManufacturer (
  manufacturerID INT IDENTITY NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  websiteURL VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive BIT NOT NULL DEFAULT '1',
  header TEXT,
  footer TEXT,
  smallImage VARCHAR(255) DEFAULT NULL,
  largeImage VARCHAR(255) DEFAULT NULL,
  isDropShipper BIT NOT NULL DEFAULT '0',
  address1 VARCHAR(255) DEFAULT NULL,
  address2 VARCHAR(255) DEFAULT NULL,
  city VARCHAR(100) DEFAULT NULL,
  state VARCHAR(100) DEFAULT NULL,
  postalCode VARCHAR(100) DEFAULT NULL,
  country VARCHAR(100) DEFAULT NULL,
  phone VARCHAR(100) DEFAULT NULL,
  email VARCHAR(100) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  manufacturerOrder INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ManufacturerBean',
  CONSTRAINT PK_manufacturerID PRIMARY KEY CLUSTERED 
  ( manufacturerID ) ON [PRIMARY],
  CONSTRAINT AK_manufacturerCode UNIQUE NONCLUSTERED 
  ( code ) ON [PRIMARY]
);

CREATE INDEX IDX_sscManufacturer_name ON sscManufacturer (name);
CREATE INDEX IDX_sscManufacturer_manufacturerOrder ON sscManufacturer (manufacturerOrder);

CREATE TABLE sscProduct (
  productID INT IDENTITY NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  seoCode VARCHAR(255) DEFAULT NULL,
  name VARCHAR(255) DEFAULT NULL,
  keywords VARCHAR(255) DEFAULT NULL,
  shortDescription VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive BIT NOT NULL DEFAULT '1',
  unitCost DECIMAL(10,4) DEFAULT NULL,
  unitPrice DECIMAL(10,4) DEFAULT NULL,
  altPrice DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  header TEXT,
  footer TEXT,
  isTaxed BIT NOT NULL DEFAULT '0',
  smallImage VARCHAR(255) DEFAULT NULL,
  mediumImage VARCHAR(255) DEFAULT NULL,
  largeImage VARCHAR(255) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  extra4 VARCHAR(255) DEFAULT NULL,
  extra5 VARCHAR(255) DEFAULT NULL,
  primaryCategoryID INT DEFAULT NULL,
  productOrder INT NOT NULL DEFAULT '0',
  created VARCHAR(100),
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductBean',
  manufacturerID INT DEFAULT NULL,
  CONSTRAINT PK_productID PRIMARY KEY CLUSTERED 
  ( productID ) ON [PRIMARY],
  CONSTRAINT AK_productCode UNIQUE NONCLUSTERED 
  ( code ) ON [PRIMARY],
  CONSTRAINT FK_product_category FOREIGN KEY (
	primaryCategoryID
  ) REFERENCES sscCategory (
	categoryID
  ) ON DELETE NO ACTION,
  CONSTRAINT FK_product_manufacturer FOREIGN KEY (
	manufacturerID
  ) REFERENCES sscManufacturer (
	manufacturerID
  ) ON DELETE NO ACTION
);

CREATE INDEX IDX_sscProduct_name ON sscProduct (name);
CREATE INDEX IDX_sscProduct_shortDescription ON sscProduct (shortDescription);
CREATE INDEX IDX_sscProduct_primaryCategoryID ON sscProduct (primaryCategoryID);
CREATE INDEX IDX_sscProduct_productOrder ON sscProduct (productOrder);

CREATE TABLE sscProductCategory (
  productCategoryID INT IDENTITY NOT NULL,
  categoryID INT NOT NULL DEFAULT '0',
  productID INT NOT NULL DEFAULT '0',
  productCategoryOrder INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductCategoryBean',
  CONSTRAINT PK_productCategoryID PRIMARY KEY CLUSTERED 
  ( productCategoryID ) ON [PRIMARY],
  CONSTRAINT AK_productCategory UNIQUE NONCLUSTERED 
  ( productID, categoryID ) ON [PRIMARY],
  CONSTRAINT FK_category_productCategory FOREIGN KEY (
	categoryID
  ) REFERENCES sscCategory (
	categoryID
  ) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FK_product_productCategory FOREIGN KEY (
	productID
  ) REFERENCES sscProduct (
	productID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE sscAttribute (
  attributeID INT IDENTITY NOT NULL,
  attributeCode VARCHAR(100) NOT NULL DEFAULT '',
  attributeName VARCHAR(255) DEFAULT NULL,
  attributeDescription TEXT,
  attributeIsActive BIT NOT NULL DEFAULT '1',
  attributeType VARCHAR(32) NOT NULL DEFAULT '',
  attributeUnitCost DECIMAL(10,4) DEFAULT NULL,
  attributeUnitPrice DECIMAL(10,4) DEFAULT NULL,
  attributeAltPrice DECIMAL(10,4) DEFAULT NULL,
  attributeWeight DECIMAL(10,4) DEFAULT NULL,
  attributeSmallImage VARCHAR(255) DEFAULT NULL,
  attributeLargeImage VARCHAR(255) DEFAULT NULL,
  defaultOptionID INT DEFAULT NULL,
  attributeIsRequired BIT NOT NULL DEFAULT '0',
  attributeOrder INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.AttributeBean',
  CONSTRAINT PK_attributeID PRIMARY KEY CLUSTERED 
  ( attributeID ) ON [PRIMARY],
  CONSTRAINT AK_attributeCode UNIQUE NONCLUSTERED 
  ( attributeCode ) ON [PRIMARY]
);

CREATE INDEX IDX_sscAttribute_code ON sscAttribute (attributeCode);
CREATE INDEX IDX_sscAttribute_name ON sscAttribute (attributeName);
CREATE INDEX IDX_sscAttribute_order ON sscAttribute (attributeOrder);

CREATE TABLE sscOption (
  optionID INT IDENTITY NOT NULL,
  attributeID INT NOT NULL DEFAULT '0',
  optionCode VARCHAR(100) NOT NULL DEFAULT '',
  optionName VARCHAR(255) DEFAULT NULL,
  optionDescription TEXT,
  optionIsActive BIT NOT NULL DEFAULT '1',
  optionUnitCost DECIMAL(10,4) DEFAULT NULL,
  optionUnitPrice DECIMAL(10,4) DEFAULT NULL,
  optionAltPrice DECIMAL(10,4) DEFAULT NULL,
  optionWeight DECIMAL(10,4) DEFAULT NULL,
  optionSmallImage VARCHAR(255) DEFAULT NULL,
  optionLargeImage VARCHAR(255) DEFAULT NULL,
  optionOrder INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.OptionBean',
  CONSTRAINT PK_optionID PRIMARY KEY CLUSTERED 
  ( optionID ) ON [PRIMARY],
  CONSTRAINT AK_optionAttribute UNIQUE NONCLUSTERED 
  ( optionCode, attributeID ) ON [PRIMARY],
  CONSTRAINT FK_option_attributeID FOREIGN KEY (
	attributeID
  ) REFERENCES sscAttribute (
	attributeID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_sscOption_code ON sscOption (optionCode);
CREATE INDEX IDX_sscOption_name ON sscOption (optionName);
CREATE INDEX IDX_sscOption_order ON sscOption (optionOrder);

CREATE TABLE sscProductAttribute (
  productAttributeID INT IDENTITY NOT NULL,
  productID INT NOT NULL DEFAULT '0',
  attributeID INT NOT NULL DEFAULT '0',
  productAttributeOrder INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductAttributeBean',
  CONSTRAINT PK_productAttributeID PRIMARY KEY CLUSTERED 
  ( productAttributeID ) ON [PRIMARY],
  CONSTRAINT AK_productAttribute UNIQUE NONCLUSTERED 
  ( productID, attributeID ) ON [PRIMARY],
  CONSTRAINT FK_product_productAttribute FOREIGN KEY (
	productID
  ) REFERENCES sscProduct (
	productID
  ) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FK_attribute_productAttribute FOREIGN KEY (
	attributeID
  ) REFERENCES sscAttribute (
	attributeID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE sscSKU (
  skuID INT IDENTITY NOT NULL,
  productID INT DEFAULT NULL,
  attributeID1 INT DEFAULT NULL,
  optionID1 INT DEFAULT NULL,
  attributeID2 INT DEFAULT NULL,
  optionID2 INT DEFAULT NULL,
  attributeID3 INT DEFAULT NULL,
  optionID3 INT DEFAULT NULL,
  attributeID4 INT DEFAULT NULL,
  optionID4 INT DEFAULT NULL,
  attributeID5 INT DEFAULT NULL,
  optionID5 INT DEFAULT NULL,
  skuNumber VARCHAR(128) DEFAULT NULL,
  skuName VARCHAR(255) DEFAULT NULL,
  skuMessage VARCHAR(255) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  trackInventory BIT NOT NULL DEFAULT '1',
  inventoryLevel INT NOT NULL DEFAULT '0',
  notes VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.SKUBean',
  CONSTRAINT PK_skuID PRIMARY KEY CLUSTERED 
  ( skuID ) ON [PRIMARY],
  CONSTRAINT AK_skuNumber UNIQUE NONCLUSTERED 
  ( skuNumber ) ON [PRIMARY],
  CONSTRAINT FK_product_sku FOREIGN KEY (
	productID
  ) REFERENCES sscProduct (
	productID
  ) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT AK_skuAll UNIQUE NONCLUSTERED 
  ( productID, attributeID1, optionID1, attributeID2, optionID2, 
  attributeID3, optionID3, attributeID4, optionID4, 
  attributeID5, optionID5 ) ON [PRIMARY]
);

CREATE TABLE sscProductSetting (
  productSettingID INT IDENTITY NOT NULL,
  productID INT NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description VARCHAR(255) DEFAULT NULL,
  type VARCHAR(100) DEFAULT NULL,
  valueType VARCHAR(16) NOT NULL,
  integerValue INT DEFAULT NULL,
  smallValue VARCHAR(64) DEFAULT NULL,
  mediumValue VARCHAR(255) DEFAULT NULL,
  textValue TEXT,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductSettingBean',
  CONSTRAINT PK_productSettingID PRIMARY KEY CLUSTERED 
  ( productSettingID ) ON [PRIMARY],
  CONSTRAINT AK_productSettingCode UNIQUE NONCLUSTERED 
  ( code, productID ) ON [PRIMARY],
  CONSTRAINT FK_product_productSetting FOREIGN KEY (
	productID
  ) REFERENCES sscProduct (
	productID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE sscCategorySetting (
  categorySettingID INT IDENTITY NOT NULL,
  categoryID INT NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description VARCHAR(255) DEFAULT NULL,
  type VARCHAR(100) DEFAULT NULL,
  valueType VARCHAR(16) NOT NULL,
  integerValue INT DEFAULT NULL,
  smallValue VARCHAR(64) DEFAULT NULL,
  mediumValue VARCHAR(255) DEFAULT NULL,
  textValue TEXT,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.CategorySettingBean',
  CONSTRAINT PK_categorySettingID PRIMARY KEY CLUSTERED 
  ( categorySettingID ) ON [PRIMARY],
  CONSTRAINT AK_categorySettingCode UNIQUE NONCLUSTERED 
  ( code, categoryID ) ON [PRIMARY],
  CONSTRAINT FK_category_categorySetting FOREIGN KEY (
	categoryID
  ) REFERENCES sscCategory (
	categoryID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE sscDiscount (
  discountID INT IDENTITY NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive BIT NOT NULL DEFAULT '1',
  usersAppliesType VARCHAR(32) DEFAULT NULL,
  usersAppliesValue VARCHAR(255) DEFAULT NULL,
  itemsAppliesType VARCHAR(32) DEFAULT NULL,
  itemsAppliesValue VARCHAR(255) DEFAULT NULL,
  cartItemsType VARCHAR(32) DEFAULT NULL,
  cartItemsValue VARCHAR(255) DEFAULT NULL,
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NULL,
  couponCode VARCHAR(100) DEFAULT NULL,
  startDate VARCHAR(100) DEFAULT NULL,
  expirationDate VARCHAR(100) DEFAULT NULL,
  timesUsed INT DEFAULT NULL,
  maxTimesUsed INT DEFAULT NULL,
  isTimesPerCustomer BIT NOT NULL DEFAULT '1',
  orderItemID INT NULL,
  discountOrder INT NOT NULL DEFAULT '0',
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountBean',
  CONSTRAINT PK_discountID PRIMARY KEY CLUSTERED 
  ( discountID ) ON [PRIMARY],
  CONSTRAINT AK_discountCode UNIQUE NONCLUSTERED 
  ( code ) ON [PRIMARY]
);

CREATE INDEX IDX_sscDiscount_name ON sscDiscount (name);
CREATE INDEX IDX_sscDiscount_discountOrder ON sscDiscount (discountOrder);

CREATE TABLE sscSKUDiscount (
  skuDiscountID INT IDENTITY NOT NULL,
  skuID INT NOT NULL DEFAULT '0',
  discountID INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.SKUDiscountBean',
  CONSTRAINT PK_skuDiscountID PRIMARY KEY CLUSTERED 
  ( skuDiscountID ) ON [PRIMARY],
  CONSTRAINT AK_skuDiscount UNIQUE NONCLUSTERED 
  ( skuID, discountID ) ON [PRIMARY],
  CONSTRAINT FK_sku_skuDiscount FOREIGN KEY (
	skuID
  ) REFERENCES sscSKU (
	skuID
  ) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FK_discount_skuDiscount FOREIGN KEY (
	discountID
  ) REFERENCES sscDiscount (
	discountID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE sscDiscountRange (
  discountRangeID INT IDENTITY NOT NULL,
  discountID INT NOT NULL,
  floor DECIMAL(10,4) NOT NULL DEFAULT '0',
  ceiling DECIMAL(10,4) NOT NULL DEFAULT '0',
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NOT NULL DEFAULT '0',
  isActive BIT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountRangeBean',
  CONSTRAINT discountRangeID PRIMARY KEY CLUSTERED 
  ( discountRangeID ) ON [PRIMARY],
  CONSTRAINT FK_discount_discountRange FOREIGN KEY (
	discountID
  ) REFERENCES sscDiscount (
	discountID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_sscDiscountRange_reward ON sscDiscountRange (reward);

CREATE TABLE sscRelatedProduct (
  relatedProductID INT IDENTITY NOT NULL,
  parentProductID INT NOT NULL DEFAULT '0',
  productID INT NOT NULL DEFAULT '0',
  relatedType VARCHAR(32) DEFAULT NULL,
  relatedProductOrder INT DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.RelatedProductBean',
  CONSTRAINT PK_relatedProductID PRIMARY KEY CLUSTERED 
  ( relatedProductID ) ON [PRIMARY],
  CONSTRAINT AK_relatedProduct UNIQUE NONCLUSTERED 
  ( parentProductID, productID ) ON [PRIMARY],
  CONSTRAINT FK_product_relatedProduct FOREIGN KEY (
	parentProductID
  ) REFERENCES sscProduct (
	productID
  ) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FK_relatedProduct_product FOREIGN KEY (
	productID
  ) REFERENCES sscProduct (
	productID
  )
);

CREATE TABLE sscProductReview (
  productReviewID INT IDENTITY NOT NULL,
  productID INT NOT NULL DEFAULT '0',
  status INT DEFAULT 0,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductReviewBean',
  nickname VARCHAR(255) DEFAULT NULL,
  firstName VARCHAR(255) DEFAULT NULL,
  lastName VARCHAR(255) DEFAULT NULL,
  email VARCHAR(255) DEFAULT NULL,
  age VARCHAR(255) DEFAULT NULL,
  city VARCHAR(255) DEFAULT NULL,
  state VARCHAR(255) DEFAULT NULL,
  review TEXT,
  rating INT,
  created VARCHAR(255) DEFAULT NULL,
  imagePath VARCHAR(255) DEFAULT NULL,
  ipaddress VARCHAR(255) DEFAULT NULL,
  country VARCHAR(255) DEFAULT NULL,
  customerID INT,
  CONSTRAINT PK_productReviewID PRIMARY KEY CLUSTERED 
  ( productReviewID ) ON [PRIMARY],
  CONSTRAINT FK_product_productReview FOREIGN KEY (
	productID
  ) REFERENCES sscProduct (
	productID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_sscProductReview_rating ON sscProductReview (rating);
CREATE INDEX IDX_sscProductReview_nickname ON sscProductReview (nickname);
