create table sscShippingMethod (
	shippingMethodID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingMethodBean', 
	code varchar(100) not null unique, 
	name varchar(255), 
	minimum float8, 
	base float8, 
	methodType varchar(255), 
	isActive bool not null, 
    numberDaysToShip int4,
    methodOrder int4,
	primary key (shippingMethodID)
);

CREATE INDEX IDX_sscShippingMethod_name ON sscShippingMethod (name);
CREATE INDEX IDX_sscSM_methodOrder ON sscShippingMethod (methodOrder);

create table sscShippingRate (
	shippingRateID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRateBean', 
	shippingMethodID int4, 
	floor float8, 
	ceiling float8, 
	amount float8, 
	isActive bool not null, 
	primary key (shippingRateID)
);

alter table sscShippingRate add constraint FKF20A6BCE3B549038 foreign key (shippingMethodID) references sscShippingMethod on delete cascade;

CREATE INDEX IDX_sscShippingRate_amount ON sscShippingRate (amount);


create table sscShippingRule (
	shippingRuleID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleBean', 
	code varchar(100) not null unique, 
	name varchar(255), 
	description text, 
	isActive bool not null, 
	ruleType varchar(32), 
	ruleOrder int4 not null, 
	extra1 varchar(255), 
	extra2 varchar(255), 
	extra3 varchar(255), 
	primary key (shippingRuleID)
);

CREATE INDEX IDX_sscShippingRule_name ON sscShippingRule (name);
CREATE INDEX IDX_sscShippingRule_ruleOrder ON sscShippingRule (ruleOrder);

create table sscSKUShippingRule (
	skuShippingRuleID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.shipping.SKUShippingRuleBean', 
	shippingRuleID int4, 
	skuID int4, 
	primary key (skuShippingRuleID)
);

alter table sscSKUShippingRule add constraint FK71A9636769AC6E foreign key (shippingRuleID) references sscShippingRule on delete cascade;
alter table sscSKUShippingRule add constraint FK71A96367E324C735 foreign key (skuID) references sscSKU on delete cascade;


create table sscShippingRuleRange (
	shippingRuleRangeID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleRangeBean', 
	shippingRuleID int4, 
	floor float8 not null, 
	ceiling float8 not null, 
	isActive bool not null, 
	discountType varchar(32), 
	discount float8, 
	pkgDescription text, 
	pkgWeight float8 not null, 
	pkgLength float8 not null, 
	pkgHeight float8 not null, 
	pkgWidth float8 not null, 
	pkgExtra1 varchar(255), 
	pkgExtra2 varchar(255), 
	pkgExtra3 varchar(255), 
	pkgExtra4 varchar(255), 
	pkgExtra5 varchar(255), 
	countries text,
	states text,
	limitMethods text, 
	limitType varchar(32), 
	primary key (shippingRuleRangeID)
);
alter table sscShippingRuleRange add constraint FK355DB57369AC6E foreign key (shippingRuleID) references sscShippingRule ON DELETE CASCADE;


create table sscOrderShippingRule (
	orderShippingRuleID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.order.OrderShippingRuleBean', 
	orderID int4, 
	orderItemID int4, 
	total float8, 
	shippingRuleID int4, 
	code varchar(100) not null, 
	name varchar(255), 
	ruleType varchar(32), 
	floor float8, 
	ceiling float8, 
	discountType varchar(32), 
	discount float8, 
	pkgDescription text, 
	pkgWeight float8 not null, 
	pkgLength float8 not null, 
	pkgHeight float8 not null, 
	pkgWidth float8 not null, 
	limitMethods text, 
	limitType varchar(32), 
	extra1 varchar(255), 
	extra2 varchar(255), 
	extra3 varchar(255), 
	pkgExtra1 varchar(255), 
	pkgExtra2 varchar(255), 
	pkgExtra3 varchar(255), 
	pkgExtra4 varchar(255), 
	pkgExtra5 varchar(255), 
	countries text,
	states text,
	subtotal float8, 
	primary key (orderShippingRuleID)
);
alter table sscOrderShippingRule add constraint FKB5F9F818DA8BA36 foreign key (orderID) references sscOrder ON DELETE CASCADE;

CREATE INDEX IDX_sscOrderShippingRule_orderItemID ON sscOrderShippingRule (orderItemID);
CREATE INDEX IDX_sscOrderShippingRule_total ON sscOrderShippingRule (total);
CREATE INDEX IDX_sscOrderShippingRule_code ON sscOrderShippingRule (code);
CREATE INDEX IDX_sscOrderShippingRule_name ON sscOrderShippingRule (name);
