package com.softslate.commerce.administrator.core;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.upload.FormFile;

import com.softslate.commerce.businessobjects.storage.FileStorageProcessor;
import com.softslate.commerce.customer.core.ActionUtils;

/**
 * @author David Tobey
 * 
 */
public class SettingsActionStoreLogo extends SettingsAction {

	static Log log = LogFactory.getLog(SettingsActionStoreLogo.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		SettingsForm baseForm = (SettingsForm) form;

		// Try uploading the store logo file
		FormFile logoFile = (FormFile) baseForm.get("storeLogoFile");
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");

		if (logoFile != null && logoFile.getFileSize() > 0) {
			if (log.isDebugEnabled())
				log.debug("Logo file being uploaded.");

			// Load the imageSystemPath and imagePathURL settings; derive the
			// imageSystemPath from the URL path if blank.
			String imageSystemPath = baseForm.getSettings().getValue(
					"imageSystemPath").toString();
			String imagePathURL = baseForm.getSettings().getValue("imagePathURL")
					.toString();
			if (!imagePathURL.endsWith("/")) {
				imagePathURL = imagePathURL + "/";
			}
			if (imageSystemPath == null || imageSystemPath.length() == 0) {
				imageSystemPath = getServlet().getServletContext().getRealPath(
						imagePathURL)
						+ System.getProperty("file.separator");
				if (!imagePathURL.startsWith("/")) {
					imagePathURL = "/" + imagePathURL;
				}
				imagePathURL = request.getContextPath() + imagePathURL;
			}
			
			String filename = actionUtils.makeFilenameNice(logoFile.getFileName());
			String fullSystemPath = imageSystemPath + filename;
			if (log.isDebugEnabled())
				log.debug("Attempting to save logo file: " + fullSystemPath);
			try {
				FileStorageProcessor fsp = (FileStorageProcessor) baseForm.getBusinessObjectFactory().createObject(
						"fileStorageProcessorImplementer");
				fsp.storeFile(logoFile.getInputStream(), fullSystemPath, "storeLogo");
				String fullURLPath = fsp.urlForFile(imagePathURL, filename, "storeLogo");
				if (log.isDebugEnabled())
					log.debug("Setting url path: " + fullURLPath);
				baseForm.set("storeLogo", fullURLPath);
			} catch (Exception e) {
				if (log.isDebugEnabled())
					log.debug("Error saving logo file: ", e);
				baseForm
						.getErrors()
						.add(
								ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage(
										"errors.detail",
										"There was an error saving the logo file to "
												+ fullSystemPath
												+ ". Please check that the directory exists on the server and that the application has write permissions to it."));
				saveErrors(request, (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}
		}

		ActionForward af = super.execute(mapping, form, request, response);
		if (log.isDebugEnabled())
			log.debug("Finishing action.");
		return af;
	}
}
