package com.softslate.commerce.administrator.extender;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.customer.core.AppUtils;

public class ExtenderCommandLine {

	static Log log = LogFactory.getLog(ExtenderCommandLine.class);

	static Properties appComponents = new Properties();
	static Properties appComponentsDefaults = new Properties();
	static Properties extenderProperties = new Properties();

	public static void main(String[] args) {
		initialize();
	}

	protected static void initialize() {
		// Initialize settings from .properties files
		try {

			// Load properties from extender-custom.properties that define what
			// we're extending and how.
			InputStream is = Thread.currentThread().getContextClassLoader()
					.getResourceAsStream("com/softslate/commerce/administrator/extender/extender-custom.properties");
			extenderProperties.load(is);

			// Find the WEB-INF directory where files will be written
			File webinfDirectory = findWEBINFDirectory(Thread.currentThread().getContextClassLoader()
					.getResource("appComponents.properties"));

			// appComponents
			Properties defaults = null;
			appComponentsDefaults = AppUtils.loadProperties("/appComponents.properties", defaults);
			appComponents = AppUtils.loadProperties("/appComponents-custom.properties", appComponentsDefaults);

			// Extension
			Extension e = new Extension(extenderProperties, appComponents, webinfDirectory);

			// Extension handler
			ExtensionHandler eh = new ExtensionHandler(e);
			eh.performExtension();

		} catch (Exception e) {
			log.error("Exception generating code: ", e);
			return;
		}

	}

	public static File findWEBINFDirectory(URL urlToAppComponentsPropertiesFile) throws Exception {
		File webinfDirectory = null;
		String webinfDirectoryPath = extenderProperties.getProperty("webinfDirectoryPath");
		if (!StringUtils.isBlank(webinfDirectoryPath)) {
			webinfDirectory = new File(webinfDirectoryPath);
		} else {
			File propertiesFile = new File(urlToAppComponentsPropertiesFile.getFile());
			File parentDirectory = new File(propertiesFile.getParent());
			webinfDirectory = new File(parentDirectory.getParent());
		}
		return webinfDirectory;
	}

}
