package com.softslate.commerce.administrator.order;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.order.CheckoutUtils;

/**
 * Struts action class corresponding to requests to
 * "/administrator/OrderEditForm.do". Loads in the current values of the record
 * being edited and populates a <code>OrderAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class OrderEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(OrderEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		OrderAddEditForm baseForm = (OrderAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		OrderProcessor processor = (OrderProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"orderProcessorImplementer");
		Map parameters = PropertyUtils.describe(form);
		Order order = null;
		if (parameters.get("orderID") != null) {
			order = processor.loadOrderFromID(parameters);
		}
		if (order == null && parameters.get("orderNumber") != null) {
			order = processor.loadOrderFromNumber(parameters);
		}
		BeanUtils.copyProperties(baseForm, order);

		// Payment forms if the order is not complete and a payment is required
		if (order.getCompleted() == null
				&& baseForm.getSettings().getValue("paymentRequired")
						.equals("1")) {

			// Do not require a confirmation screen
			Settings alteredSettings = baseForm.getBusinessObjectFactory()
					.getSettings().copyAndAlter("displayCheckoutConfirm", "0");
			baseForm.getRequest().setAttribute("settings", alteredSettings);

			// Prepare payment forms
			CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"checkoutUtilsImplementer");
			checkoutUtils.preparePaymentForms(baseForm, getServlet());
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("orderEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Orders",
				"Order Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("order", order);
		request.setAttribute("orderAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
