package com.softslate.commerce.administrator.payment;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.businessobjects.payment.AdminPaymentProcessor;
import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/PaymentEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>PaymentAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class PaymentEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(PaymentAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		PaymentAddEditForm baseForm = (PaymentAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		AdminPaymentProcessor processor = (AdminPaymentProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"adminPaymentProcessorImplementer");
		Payment payment = processor.loadPaymentFromID(PropertyUtils
				.describe(form));
		BeanUtils.copyProperties(baseForm, payment);

		// Load the parent if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		Order order = null;
		if (baseForm.getOrderID() != null && baseForm.getOrderID().length() > 0) {
			OrderProcessor parentProcessor =  bof
					.createObject(OrderProcessor.class);
			order = parentProcessor.loadOrderFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("order", order);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("orderPaymentEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Orders",
				"Order Details", "Payments", "Payment Details" });
		request.setAttribute("breadcrumbs", bcs);
		
		request.setAttribute("payment", payment);
		request.setAttribute("paymentAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
