package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/AttributeAdd.do". Processes a request to add a record to the
 * system.
 * 
 * @author David Tobey
 * 
 */
public class AttributeAddAction extends BaseAction {

	static Log log = LogFactory.getLog(AttributeAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Add the record
		AttributeAddEditForm baseForm = (AttributeAddEditForm) form;

		// Handle generating the code from the name if it is empty
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		if ((baseForm.getAttributeCode() == null || baseForm.getAttributeCode()
				.equals(""))
				&& (baseForm.getAttributeName() != null && !baseForm
						.getAttributeName().equals(""))) {
			baseForm.setAttributeCode(actionUtils.generateCodeFromName(baseForm
					.getAttributeName()));
		}

		AttributeProcessor processor = (AttributeProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"attributeProcessorImplementer");
		Map results = processor.addAttribute(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null
				|| results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Attribute newRecord = (Attribute) results.get("newRecord");

				// entity.jsp wants the new record in the request scope.
				request.setAttribute("attribute", newRecord);
				baseForm.getMessages().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.editNewRecord",
								"AttributeEditForm.do?attributeID="
										+ newRecord.getAttributeID()));
				if (newRecord.getAttributeType() != null
						&& (newRecord.getAttributeType().equals("compact") || newRecord
								.getAttributeType().equals("expanded"))) {
					String params = "";
					if (request.getParameter("productID") != null) {
						params = params + "&productID="
								+ request.getParameter("productID");
					}
					baseForm.getMessages().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.detail",
									"<a href=\"Option.do?attributeID="
											+ newRecord.getAttributeID()
											+ params
											+ "\">Add options under this "
											+ "attribute</a>."));
				}
			}
			saveMessages(baseForm.getRequest(), baseForm.getMessages());

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");

			return mapping.findForward("success");
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			Attribute existingRecord = (Attribute) results
					.get("existingRecord");
			String existingCode = existingRecord.getAttributeCode();
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode"
							+ results.get("resultCode"), results
							.get("duplicateField"), existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord",
							"AttributeEditForm.do?attributeID="
									+ existingRecord.getAttributeID(), results
									.get("duplicateField"), existingCode));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
		}

		return mapping.findForward("success");
	}
}
