package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/ManufacturerEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>ManufacturerAddEditForm</code>
 * object.
 * 
 * @author David Tobey
 * 
 */
public class ManufacturerEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(ManufacturerEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ManufacturerAddEditForm baseForm = (ManufacturerAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		ManufacturerProcessor processor = (ManufacturerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"manufacturerProcessorImplementer");
		Manufacturer manufacturer = null;
		if (baseForm.getManufacturerID() != null) {
			manufacturer = processor.loadManufacturerFromID(PropertyUtils.describe(form));
		}
		if (manufacturer == null && baseForm.getCode() != null) {
			manufacturer = processor.loadManufacturerFromCode(PropertyUtils.describe(form));
		}
		BeanUtils.copyProperties(baseForm, manufacturer);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("manufacturerEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Manufacturers",
				"Manufacturer Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("manufacturer", manufacturer);
		request.setAttribute("manufacturerAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
