package com.softslate.commerce.businessobjects.administrator;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface for business logic methods having to do with administrators.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting administrators.
 * <p>
 * When the application needs to create an instance that implements
 * <code>AdministratorProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "administratorProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "administratorProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.administrator.BasicAdministratorProcessor}.
 * 
 * @author David Tobey
 */
public interface AdministratorProcessor extends BusinessObject {

	abstract Map processLogin(Map parameters) throws Exception;

	abstract Map processApiLogin(String apiLoginID, String apiAuthenticationToken) throws Exception;

	abstract Map loadAdministratorsAndCount(Map parameters) throws Exception;

	abstract Map addAdministrator(Map parameters) throws Exception;

	abstract Administrator loadAdministratorFromID(Map parameters) throws Exception;

	abstract Map editAdministrator(Map parameters) throws Exception;

	abstract Map editDeleteAdministrators(Map parameters) throws Exception;

	abstract void deleteAdministrator(Map parameters) throws Exception;

	abstract void administratorChangePassword(String newDecryptedPassword, Administrator administrator)
			throws Exception;

	abstract Map processChangePassword(String password, String newDecryptedPassword, String userName) throws Exception;

}
