package com.softslate.commerce.businessobjects.core;

import java.util.Collection;
import java.util.Map;

/**
 * Interface for business logic methods having to do with request logs. Created
 * in the Struts layer in response to requests for viewing, editing, adding, and
 * deleting request logs.
 * <p>
 * When the application needs to create an instance that implements
 * <code>RequestLogProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "requestLogProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "requestLogProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.core.BasicRequestLogProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface RequestLogProcessor extends BusinessProcessor {

	abstract Map loadRequestLogsAndCount(Map parameters) throws Exception;

	abstract Map addRequestLog(Map parameters) throws Exception;

	abstract RequestLog loadRequestLogFromID(Map parameters) throws Exception;

	abstract Map editRequestLog(Map parameters) throws Exception;

	abstract Map updateRequestLog(RequestLog requestLog) throws Exception;
	
	abstract Map editDeleteRequestLogs(Map parameters) throws Exception;

	abstract void deleteRequestLog(Map parameters) throws Exception;

	abstract Map postChangingRequestLogEvent(RequestLog requestLog, Map parameters) throws Exception;

	abstract Collection loadRequestLogStatistics() throws Exception;

	abstract int pruneRequestLog(Map parameters) throws Exception;

}
