package com.softslate.commerce.businessobjects.customer;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;
import com.softslate.commerce.businessobjects.product.Product;

/**
 * Interface for business logic methods having to do with customer wish lists.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting wish lists.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CustomerWishListProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "customerWishListProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "customerWishListProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.customer.BasicCustomerWishListProcessor}.
 * 
 * @author Jason McWilliams
 * 
 */
public interface CustomerWishListProcessor extends BusinessProcessor {

	abstract Map loadCustomerWishListsAndCount(Map parameters) throws Exception;

	abstract Map addCustomerWishList(Map parameters) throws Exception;

	abstract CustomerWishList loadCustomerWishListFromID(Map parameters)
			throws Exception;

	abstract Map editCustomerWishList(Map parameters) throws Exception;

	abstract Map editDeleteCustomerWishLists(Map parameters) throws Exception;

	abstract void deleteCustomerWishList(Map parameters) throws Exception;

	abstract Map customerDeleteWishList(Map parameters) throws Exception;

	abstract Map customerEditWishList(Map parameters) throws Exception;

	abstract CustomerWishList customerGetWishList(Map parameters, boolean createNew)
			throws Exception;

	abstract Map customerDeleteWishListProduct(Map parameters) throws Exception;

	abstract Map customerAddProductToWishList(
			CustomerWishList customerWishList, Product product)
			throws Exception;

	abstract Map customerAddWishList(Map parameters) throws Exception;

	abstract String computeCheck(CustomerWishList customerWishList);
}
