package com.softslate.commerce.businessobjects.email;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.email.TemplateDAO;
import com.softslate.commerce.daos.email.TemplateGatewayDAO;

public class BasicTemplateProcessor extends BaseBusinessProcessor implements TemplateProcessor {

    static Log log = LogFactory.getLog(BasicTemplateProcessor.class);

    public Map loadTemplatesAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Loading Templates and count");
        TemplateGatewayDAO templateDAO =  getDaoFactory().createDAO(TemplateGatewayDAO.class);
        return templateDAO.loadTemplatesAndCount(parameters);
    }

    public Map addTemplate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Adding Template");
        Map results = new HashMap();
        Template template =  getBusinessObjectFactory().createObject(Template.class);
        BeanUtils.copyProperties(template, parameters);
        TemplateDAO templateDAO =  getDaoFactory().createDAO(TemplateDAO.class);
        templateDAO.setTemplate(template);
        templateDAO.insertTemplate(true);
        results.put("newRecord", templateDAO.getTemplate());
        return results;
    }

    public Template loadTemplateFromID(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Loading Template from id");
        Template template =  getBusinessObjectFactory().createObject(Template.class);
        TemplateDAO templateDAO =  getDaoFactory().createDAO(TemplateDAO.class);
        BeanUtils.copyProperties(template, parameters);
        templateDAO.setTemplate(template);
        templateDAO.loadTemplateFromID();
        return templateDAO.getTemplate();
    }

    public Map editTemplate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Editing Template");
        Template template = loadTemplateFromID(parameters);
        BeanUtils.copyProperties(template, parameters);
        TemplateDAO templateDAO =  getDaoFactory()
                .createDAO(TemplateDAO.class);
        templateDAO.setTemplate(template);
        templateDAO.updateTemplate(true);
        Map results = new HashMap();
        results.put("updatedRecord", templateDAO.getTemplate());
        return results;
    }

    public void editDeleteTemplates(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple Templates");
        TemplateGatewayDAO templateDAO =  getDaoFactory().createDAO(TemplateGatewayDAO.class);
        templateDAO.editDeleteTemplates(parameters, getBusinessObjectFactory());
        return;
    }

    public void deleteTemplate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Deleting Template");
        Template template =  getBusinessObjectFactory().createObject(Template.class);
        BeanUtils.copyProperties(template, parameters);
        TemplateDAO templateDAO =  getDaoFactory().createDAO(TemplateDAO.class);
        templateDAO.setTemplate(template);
        templateDAO.deleteTemplate(true);
        return;
    }

	public Collection<Template> loadAllTemplates() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading all Templates");
		TemplateGatewayDAO gatewayDAO =  getDaoFactory().createDAO(TemplateGatewayDAO.class);
		return gatewayDAO.loadAllTemplates();
	}

	public Template loadTemplateFromCode(String code) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading template from code: " + code);
		TemplateGatewayDAO gatewayDAO =  getDaoFactory().createDAO(TemplateGatewayDAO.class);
		return gatewayDAO.loadTemplateFromCode(code);
	}
}
