package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.order.OrderDiscountDAO;
import com.softslate.commerce.daos.order.OrderDiscountGatewayDAO;

/**
 * Processes business logic having to do with order discounts. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting order discounts.
 * <p>
 * <code>BasicOrderDiscountProcessor</code> is the default implementation of
 * the <code>OrderDiscountProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicOrderDiscountProcessor extends BaseBusinessProcessor
        implements OrderDiscountProcessor {

    static Log log = LogFactory.getLog(BasicOrderDiscountProcessor.class);

    public Map loadOrderDiscountsAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading order discounts and count");
        OrderDiscountGatewayDAO gatewayDAO =  getDaoFactory()
                .createDAO(OrderDiscountGatewayDAO.class);
        return gatewayDAO.loadOrderDiscountsAndCount(parameters);
    }

    public Map addOrderDiscount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding order discount: " + code);
        }
        OrderDiscount orderDiscount =  getBusinessObjectFactory()
                .createObject(OrderDiscount.class);
        BeanUtils.copyProperties(orderDiscount, parameters);
        Order order = (Order) getBusinessObjectFactory().createObject(
                "orderImplementer");
        order.setOrderID(orderDiscount.getOrderID());
        orderDiscount.setOrder(order);
        if (orderDiscount.getOrderItemID() != null
                && orderDiscount.getOrderItemID().intValue() != 0) {
            OrderItem orderItem =  getBusinessObjectFactory()
                    .createObject(OrderItem.class);
            orderItem.setOrderItemID(orderDiscount.getOrderItemID().intValue());
            orderDiscount.setOrderItem(orderItem);
        }
        OrderDiscountDAO dao = (OrderDiscountDAO) getDaoFactory().createDAO(
                "orderDiscountDAOImplementer");
        dao.setOrderDiscount(orderDiscount);
        dao.insertOrderDiscount(true);
        return null;
    }

    public OrderDiscount loadOrderDiscountFromID(Map parameters)
            throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading order discount from id");
        OrderDiscount orderDiscount =  getBusinessObjectFactory()
                .createObject(OrderDiscount.class);
        OrderDiscountDAO dao = (OrderDiscountDAO) getDaoFactory().createDAO(
                "orderDiscountDAOImplementer");
        BeanUtils.copyProperties(orderDiscount, parameters);
        dao.setOrderDiscount(orderDiscount);
        dao.loadOrderDiscountFromID();
        return orderDiscount;
    }

    public Map editOrderDiscount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing order discount: " + code);
        }
        OrderDiscount orderDiscount =  getBusinessObjectFactory()
                .createObject(OrderDiscount.class);
        BeanUtils.copyProperties(orderDiscount, parameters);
        Order order = (Order) getBusinessObjectFactory().createObject(
                "orderImplementer");
        order.setOrderID(orderDiscount.getOrderID());
        orderDiscount.setOrder(order);
        if (orderDiscount.getOrderItemID() != null
                && orderDiscount.getOrderItemID().intValue() != 0) {
            OrderItem orderItem =  getBusinessObjectFactory()
                    .createObject(OrderItem.class);
            orderItem.setOrderItemID(orderDiscount.getOrderItemID().intValue());
            orderDiscount.setOrderItem(orderItem);
        }
        OrderDiscountDAO dao = (OrderDiscountDAO) getDaoFactory().createDAO(
                "orderDiscountDAOImplementer");
        dao.setOrderDiscount(orderDiscount);
        dao.updateOrderDiscount(true);
        return null;
    }

	public Map editDeleteOrderDiscounts(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple order discounts");
		OrderDiscountGatewayDAO dao = (OrderDiscountGatewayDAO) getDaoFactory().createDAO(
				"orderDiscountGatewayDAOImplementer");
		Map results = dao.editDeleteOrderDiscounts(parameters, getBusinessObjectFactory());
		return results;
	}

    public void deleteOrderDiscount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Delete order discount: " + code);
        }
        OrderDiscount orderDiscount =  getBusinessObjectFactory()
                .createObject(OrderDiscount.class);
        BeanUtils.copyProperties(orderDiscount, parameters);
        OrderDiscountDAO dao = (OrderDiscountDAO) getDaoFactory().createDAO(
                "orderDiscountDAOImplementer");
        dao.setOrderDiscount(orderDiscount);
        dao.deleteOrderDiscount(true);
        return;
    }
}
