package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with order deliveries.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting order deliveries.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderDeliveryProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderDeliveryProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderDeliveryProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderDeliveryProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface OrderDeliveryProcessor extends BusinessProcessor {

	abstract Map loadOrderDeliveriesAndCount(Map parameters) throws Exception;

	abstract Map addOrderDelivery(Map parameters) throws Exception;

	abstract OrderDelivery loadOrderDeliveryFromID(Map parameters)
			throws Exception;

	abstract Map editOrderDelivery(Map parameters) throws Exception;

	abstract Map editDeleteOrderDeliveries(Map parameters) throws Exception;

	abstract void deleteOrderDelivery(Map parameters) throws Exception;

	abstract OrderDelivery loadOrderDeliveryFromID(int orderDeliveryID) throws Exception;

	abstract Map postChangingOrderDeliveryEvent(OrderDelivery orderDelivery, Map parameters) throws Exception;

}
