package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.daos.payment.PaymentDAO;

/**
 * Processes a check or money order payment for the store. Simply records the
 * fact that the user elected to by pay with a check or money order, so that the
 * order can be looked up and processed after payment is received.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.daos.payment.PaymentDAO
 */
public class CheckPaymentProcessor extends BasePaymentProcessor implements
		PaymentProcessor {

	static Log log = LogFactory.getLog(CheckPaymentProcessor.class);

	public Map process(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing payment.");
		String checkPaymentProcessorMessage = (String) getSettings().getValue(
				"checkPaymentProcessorMessage");
		if (checkPaymentProcessorMessage == null) {
			checkPaymentProcessorMessage = "User elected to pay by check.";
		}
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		BeanUtils.copyProperties(payment, parameters);
		payment.setStatus("Pending");
		payment.setAmount(getUser().getOrder().getTotal());
		payment.setCreated(formatDateTime(new java.util.Date()));
		payment.setOrder(getUser().getOrder());
		payment.setProcessorClassName(this.getClass().getName());
		payment.setValue4(checkPaymentProcessorMessage);
		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		paymentDAO.setPayment(payment);
		paymentDAO.insertPayment(false);
		if (getUser().getOrder().getPayments() == null) {
			getUser().getOrder().setPayments(new LinkedHashSet());
		}
		getUser().getOrder().getPayments().add(payment);

		Map result = new HashMap();
		result.put("resultCode", "0");
		return result;
	}
}
