package com.softslate.commerce.businessobjects.shipping;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.shipping.ShippingRuleRangeDAO;
import com.softslate.commerce.daos.shipping.ShippingRuleRangeGatewayDAO;

/**
 * Processes business logic having to do with shipping rule ranges. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting shipping rule ranges.
 * <p>
 * <code>BasicShippingRuleRangeProcessor</code> is the default implementation of
 * the <code>ShippingRuleRangeProcessor</code> interface for the application.
 * 
 * @author Jason McWilliams
 */
public class BasicShippingRuleRangeProcessor extends BaseBusinessProcessor
        implements ShippingRuleRangeProcessor {

    static Log log = LogFactory.getLog(BasicShippingRuleRangeProcessor.class);

    public Map loadShippingRuleRangesAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading shipping rule ranges and count");
        ShippingRuleRangeGatewayDAO gatewayDAO =  getDaoFactory()
                .createDAO(ShippingRuleRangeGatewayDAO.class);
        return gatewayDAO.loadShippingRuleRangesAndCount(parameters);
    }

    public Map addShippingRuleRange(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String shippingRuleRangeID = (String) parameters.get("shippingRuleRangeID");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding shipping rule range: " + shippingRuleRangeID);
        }
		Map results = new HashMap();
        ShippingRuleRange shippingRuleRange =  getBusinessObjectFactory()
                .createObject(ShippingRuleRange.class);
        BeanUtils.copyProperties(shippingRuleRange, parameters);
        ShippingRule shippingRule = (ShippingRule) getBusinessObjectFactory().createObject(
                "shippingRuleImplementer");
        shippingRule.setShippingRuleID(shippingRuleRange.getShippingRuleID());
        shippingRuleRange.setShippingRule(shippingRule);
        ShippingRuleRangeDAO dao = (ShippingRuleRangeDAO) getDaoFactory().createDAO(
                "shippingRuleRangeDAOImplementer");
        dao.setShippingRuleRange(shippingRuleRange);
        dao.insertShippingRuleRange(true);
		results.put("newRecord", dao.getShippingRuleRange());
		return results;
    }

    public ShippingRuleRange loadShippingRuleRangeFromID(Map parameters)
            throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading shipping rule range from id");
        ShippingRuleRange shippingRuleRange =  getBusinessObjectFactory()
                .createObject(ShippingRuleRange.class);
        ShippingRuleRangeDAO dao = (ShippingRuleRangeDAO) getDaoFactory().createDAO(
                "shippingRuleRangeDAOImplementer");
        BeanUtils.copyProperties(shippingRuleRange, parameters);
        dao.setShippingRuleRange(shippingRuleRange);
        dao.loadShippingRuleRangeFromID();
		return dao.getShippingRuleRange();
    }

    public Map editShippingRuleRange(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String shippingRuleRangeID = (String) parameters.get("shippingRuleRangeID");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing shipping rule range: " + shippingRuleRangeID);
        }
		Map results = new HashMap();
		ShippingRuleRange shippingRuleRange = loadShippingRuleRangeFromID(parameters);
        BeanUtils.copyProperties(shippingRuleRange, parameters);
        ShippingRule shippingRule = (ShippingRule) getBusinessObjectFactory().createObject(
                "shippingRuleImplementer");
        shippingRule.setShippingRuleID(shippingRuleRange.getShippingRuleID());
        shippingRuleRange.setShippingRule(shippingRule);
        ShippingRuleRangeDAO dao = (ShippingRuleRangeDAO) getDaoFactory().createDAO(
                "shippingRuleRangeDAOImplementer");
        dao.setShippingRuleRange(shippingRuleRange);
        dao.updateShippingRuleRange(true);
		results.put("updatedRecord", dao.getShippingRuleRange());
		return results;
    }

	public Map editDeleteShippingRuleRanges(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple shipping rule ranges");
		ShippingRuleRangeGatewayDAO dao = (ShippingRuleRangeGatewayDAO) getDaoFactory().createDAO(
				"shippingRuleRangeGatewayDAOImplementer");
		Map results = dao.editDeleteShippingRuleRanges(parameters, getBusinessObjectFactory());
		return results;
	}

    public void deleteShippingRuleRange(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String shippingRuleRangeID = (String) parameters.get("shippingRuleRangeID");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Deleting shipping rule range: " + shippingRuleRangeID);
        }
        ShippingRuleRange shippingRuleRange =  getBusinessObjectFactory()
                .createObject(ShippingRuleRange.class);
        BeanUtils.copyProperties(shippingRuleRange, parameters);
        ShippingRuleRangeDAO dao = (ShippingRuleRangeDAO) getDaoFactory().createDAO(
                "shippingRuleRangeDAOImplementer");
        dao.setShippingRuleRange(shippingRuleRange);
        dao.deleteShippingRuleRange(true);
        return;
    }
}
