package com.softslate.commerce.businessobjects.shipping;

import java.io.Serializable;
import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a discount within the system. <code>ShippingRuleBean</code>
 * is the default implementation of the <code>ShippingRule</code> interface for
 * the application.
 * <p>
 * The database stores discounts in the <code>sscShippingRule</code> table, and
 * this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>ShippingRuleBean.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author Jason McWilliams
 */
@Audited
@AuditTable("AudShippingRule")
public class ShippingRuleBean extends BaseBusinessObject implements ShippingRule,
		Serializable {

	private static final long serialVersionUID = 7325371793592390886L;
	private int shippingRuleID = 0;
	private String code = null;
	private String name = null;
	private String description = null;
	private boolean isActive = true;
	private String ruleType = null;
	private int ruleOrder = 0;
	private String extra1 = null;
	private String extra2 = null;
	private String extra3 = null;
	private Collection skuShippingRules = null;
	private Collection shippingRuleRanges = null;

	
	public int getShippingRuleID() {
		return shippingRuleID;
	}
	public void setShippingRuleID(int shippingRuleID) {
		this.shippingRuleID = shippingRuleID;
	}
	
	public String getCode() {
		return code;
	}
	public void setCode(String code) {
		this.code = code;
	}
	
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	
	public boolean getIsActive() {
		return isActive;
	}
	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}
	
	public String getRuleType() {
		return ruleType;
	}
	public void setRuleType(String ruleType) {
		this.ruleType = ruleType;
	}

	public int getRuleOrder() {
		return ruleOrder;
	}
	public void setRuleOrder(int ruleOrder) {
		this.ruleOrder = ruleOrder;
	}

	public String getExtra1() {
		return extra1;
	}
	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public String getExtra2() {
		return extra2;
	}
	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}
	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public Collection getSkuShippingRules() {
		return skuShippingRules;
	}
	public void setSkuShippingRules(Collection skuShippingRules) {
		this.skuShippingRules = skuShippingRules;
	}

	public Collection getShippingRuleRanges() {
		return shippingRuleRanges;
	}
	public void setShippingRuleRanges(Collection shippingRuleRanges) {
		this.shippingRuleRanges = shippingRuleRanges;
	}

}
