package com.softslate.commerce.businessobjects.tax;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface for business logic methods having to do with basic tax rates.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting basic tax rates.
 * <p>
 * When the application needs to create an instance that implements
 * <code>TaxRateProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "taxRateProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "taxRateProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.tax.BasicTaxRateProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface TaxRateProcessor extends BusinessObject {

	abstract Map loadBasicTaxRatesAndCount(Map parameters) throws Exception;

	abstract Map addTaxRate(Map parameters) throws Exception;

	abstract BasicTaxRate loadBasicTaxRateFromID(Map parameters)
			throws Exception;

	abstract Map editTaxRate(Map parameters) throws Exception;

	abstract Map editDeleteTaxRates(Map parameters) throws Exception;

	abstract void deleteTaxRate(Map parameters) throws Exception;
}
