package com.softslate.commerce.customer.core;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.util.ByteArrayDataSource;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.ApplicationException;
import com.softslate.commerce.businessobjects.core.ApplicationMessage;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.BusinessObjectUtils;
import com.softslate.commerce.businessobjects.core.RequestLog;
import com.softslate.commerce.businessobjects.core.RequestLogProcessor;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.CartDiscountProcessor;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderItemAttribute;
import com.softslate.commerce.businessobjects.shipping.ShippingRuleProcessor;
import com.softslate.commerce.customer.order.CheckoutUtils;

/**
 * <code>ActionUtilsImpl</code> is the default implementation of the
 * <code>ActionUtils</code> interface for the application.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.customer.core.ActionUtils
 * 
 */
public class ActionUtilsImpl implements ActionUtils {

	static Log log = LogFactory.getLog(ActionUtilsImpl.class);
	static Log customerAuditLog = LogFactory
			.getLog("com.softslate.commerce.customer.AuditLog");
	static Log adminAuditLog = LogFactory
			.getLog("com.softslate.commerce.administrator.AuditLog");

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.ActionUtils#parseGenericTemplate
	 * (com.softslate.commerce.customer.core.BaseForm, java.lang.String)
	 */
	public String parseGenericTemplate(BaseForm baseForm, String template) {
		StringBuffer result = new StringBuffer(template);
		User activeUser = null;
		if (baseForm.getRequest() != null) {
			activeUser = (User) baseForm.getRequest().getAttribute("user");
		}
		if (activeUser == null) {
			activeUser = baseForm.getUser();
		}
		String currencyCode = (String) baseForm.getSettings().getValue(
				"currencyCode");
		Locale locale = activeUser.getLocale();
		BusinessObjectUtils utils = (BusinessObjectUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"businessObjectUtilsImplementer");
		try {
			// LOGIN_URL
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("customerSecureURL") + "/Account.do",
					"%%LOGIN_URL%%");
			// SITE_COMPANY
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeName"), "%%SITE_COMPANY%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeName"), "%%STORE_NAME%%");
			// SITE_ADDRESS
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeAddress1"), "%%SITE_ADDRESS%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeAddress1"), "%%STORE_ADDRESS%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeAddress1"), "%%STORE_ADDRESS1%%");
			// %%STORE_ADDRESS2%%
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeAddress2"), "%%STORE_ADDRESS2%%");
			// SITE_CITY
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeCity"), "%%SITE_CITY%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeCity"), "%%STORE_CITY%%");
			// SITE_STATE
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeStateOrProvince"), "%%SITE_STATE%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeStateOrProvince"), "%%STORE_STATE%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeStateOrProvince"),
					"%%STORE_STATEORPROVINCE%%");
			// SITE_ZIP
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storePostalCode"), "%%SITE_ZIP%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storePostalCode"), "%%STORE_ZIP%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storePostalCode"), "%%STORE_POSTALCODE%%");
			// SITE_COUNTRY
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeCountry"), "%%SITE_COUNTRY%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeCountry"), "%%STORE_COUNTRY%%");
			// SITE_PHONE
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storePhone"), "%%SITE_PHONE%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storePhone"), "%%STORE_PHONE%%");
			// SITE_EMAIL
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeEmail"), "%%SITE_EMAIL%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeEmail"), "%%STORE_EMAIL%%");
			// SITE_FAX
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeFax"), "%%SITE_FAX%%");
			result = replaceValue(result, (String) baseForm.getSettings()
					.getValue("storeFax"), "%%STORE_FAX%%");
			if (activeUser != null && activeUser.getOrder() != null) {
				// ORDER_ID
				result = replaceValue(result,
						Integer.toString(activeUser.getOrder().getOrderID()),
						"%%ORDER_ID%%");
				// CUSTOMER_ID
				String value = "";
				if (activeUser.getOrder().getCustomerID() != null) {
					value = activeUser.getOrder().getCustomerID().toString();
				}
				result = replaceValue(result, value, "%%CUSTOMER_ID%%");
				// USER_NAME
				result = replaceValue(result, activeUser.getOrder()
						.getUserName(), "%%USER_NAME%%");
				// INVOICE_NUM
				value = "";
				if (activeUser.getOrder().getOrderNumber() != null) {
					value = activeUser.getOrder().getOrderNumber().toString();
				}
				result = replaceValue(result, value, "%%INVOICE_NUM%%");
				result = replaceValue(result, value, "%%ORDER_NUM%%");
				result = replaceValue(result, value, "%%ORDER_NUMBER%%");
				// ORDER_TOTAL
				result = replaceValue(result, utils.formatPrice(activeUser
						.getOrder().getTotal(), currencyCode, locale),
						"%%ORDER_TOTAL%%");
				// SHIPPING
				result = replaceValue(result, utils.formatPrice(activeUser
						.getOrder().getShipping(), currencyCode, locale),
						"%%SHIPPING%%");
				result = replaceValue(result, utils.formatPrice(activeUser
						.getOrder().getShipping(), currencyCode, locale),
						"%%ORDER_SHIPPING%%");
				// TAX
				result = replaceValue(result, utils.formatPrice(activeUser
						.getOrder().getTax(), currencyCode, locale), "%%TAX%%");
				result = replaceValue(result, utils.formatPrice(activeUser
						.getOrder().getTax(), currencyCode, locale),
						"%%ORDER_TAX%%");
				// ORDER_SUBTOTAL
				result = replaceValue(result, utils.formatPrice(activeUser
						.getOrder().getSubtotal(), currencyCode, locale),
						"%%ORDER_SUBTOTAL%%");
				// ORDER_TAXABLESUBTOTAL
				result = replaceValue(result,
						utils.formatPrice(activeUser.getOrder()
								.getTaxableSubtotal(), currencyCode, locale),
						"%%ORDER_TAXABLESUBTOTAL%%");
				// ORDER_DATE
				result = replaceValue(
						result,
						formatDate(activeUser.getOrder().getCompleted(), locale),
						"%%ORDER_DATE%%");
				// ORDER_LASTMODIFIED
				result = replaceValue(
						result,
						formatDate(activeUser.getOrder().getLastModified(),
								locale), "%%ORDER_LASTMODIFIED%%");
				// ORDER_STATUS
				result = replaceValue(result,
						activeUser.getOrder().getStatus(), "%%ORDER_STATUS%%");
				// ORDER_STATUSDETAILS
				result = replaceValue(result, activeUser.getOrder()
						.getStatusDetails(), "%%ORDER_STATUSDETAILS%%");
				// ORDER_TRACKINGNUMBER
				result = replaceValue(result, activeUser.getOrder()
						.getTrackingNumber(), "%%ORDER_TRACKINGNUMBER%%");
				// ORDER_WEIGHT
				value = "";
				if (activeUser.getOrder().getWeight() != null) {
					value = activeUser.getOrder().getWeight().toString();
				}
				result = replaceValue(result, value, "%%ORDER_WEIGHT%%");
				// ORDER_QUANTITY
				value = "";
				if (activeUser.getOrder().getQuantity() != null) {
					value = activeUser.getOrder().getQuantity().toString();
				}
				result = replaceValue(result, value, "%%ORDER_QUANTITY%%");
				// BILL_FNAME
				result = replaceValue(result, activeUser.getOrder()
						.getFirstName(), "%%BILL_FNAME%%");
				result = replaceValue(result, activeUser.getOrder()
						.getFirstName(), "%%BILL_FIRSTNAME%%");
				// BILL_LNAME
				result = replaceValue(result, activeUser.getOrder()
						.getLastName(), "%%BILL_LNAME%%");
				result = replaceValue(result, activeUser.getOrder()
						.getLastName(), "%%BILL_LASTNAME%%");
				// BILL_COMP
				result = replaceValue(result, activeUser.getOrder()
						.getOrganization(), "%%BILL_COMP%%");
				result = replaceValue(result, activeUser.getOrder()
						.getOrganization(), "%%BILL_COMPANY%%");
				result = replaceValue(result, activeUser.getOrder()
						.getOrganization(), "%%BILL_ORGANIZATION%%");
				// BILL_ADDR
				result = replaceValue(result, activeUser.getOrder()
						.getAddress1(), "%%BILL_ADDR%%");
				result = replaceValue(result, activeUser.getOrder()
						.getAddress1(), "%%BILL_ADDRESS%%");
				result = replaceValue(result, activeUser.getOrder()
						.getAddress1(), "%%BILL_ADDRESS1%%");
				// BILL_ADDR2
				result = replaceValue(result, activeUser.getOrder()
						.getAddress2(), "%%BILL_ADDR2%%");
				result = replaceValue(result, activeUser.getOrder()
						.getAddress2(), "%%BILL_ADDRESS2%%");
				// BILL_CITY
				result = replaceValue(result, activeUser.getOrder().getCity(),
						"%%BILL_CITY%%");
				// BILL_STATE
				result = replaceValue(result, activeUser.getOrder().getState(),
						"%%BILL_STATE%%");
				// BILL_ZIP
				result = replaceValue(result, activeUser.getOrder()
						.getPostalCode(), "%%BILL_ZIP%%");
				result = replaceValue(result, activeUser.getOrder()
						.getPostalCode(), "%%BILL_POSTALCODE%%");
				// BILL_CNTRY
				result = replaceValue(result, activeUser.getOrder()
						.getCountry(), "%%BILL_CNTRY%%");
				result = replaceValue(result, activeUser.getOrder()
						.getCountry(), "%%BILL_COUNTRY%%");
				// BILL_PHONE
				result = replaceValue(result,
						activeUser.getOrder().getPhone1(), "%%BILL_PHONE%%");
				result = replaceValue(result,
						activeUser.getOrder().getPhone1(), "%%BILL_PHONE1%%");
				// BILL_PHONE2
				result = replaceValue(result,
						activeUser.getOrder().getPhone2(), "%%BILL_PHONE2%%");
				// BILL_EMAIL
				result = replaceValue(result,
						activeUser.getOrder().getEmail1(), "%%BILL_EMAIL%%");
				result = replaceValue(result,
						activeUser.getOrder().getEmail1(), "%%BILL_EMAIL1%%");
				// BILL_EMAIL2
				result = replaceValue(result,
						activeUser.getOrder().getEmail2(), "%%BILL_EMAIL2%%");
				// BILL_EXTRA
				result = replaceValue(result,
						activeUser.getOrder().getExtra1(), "%%BILL_EXTRA%%");
				result = replaceValue(result,
						activeUser.getOrder().getExtra1(), "%%BILL_EXTRA1%%");
				// BILL_EXTRA2
				result = replaceValue(result,
						activeUser.getOrder().getExtra2(), "%%BILL_EXTRA2%%");
				// BILL_EXTRA3
				result = replaceValue(result,
						activeUser.getOrder().getExtra3(), "%%BILL_EXTRA3%%");
				// BILL_EXTRA4
				result = replaceValue(result,
						activeUser.getOrder().getExtra4(), "%%BILL_EXTRA4%%");
				// BILL_EXTRA5
				result = replaceValue(result,
						activeUser.getOrder().getExtra5(), "%%BILL_EXTRA5%%");
				// BILL_NOTES
				result = replaceValue(result, activeUser.getOrder().getNotes(),
						"%%BILL_NOTES%%");
			}
			if (activeUser != null && activeUser.getOrder() != null
					&& activeUser.getOrder().firstDelivery() != null) {
				// DELIVERY_NUMBER
				result = replaceValue(
						result,
						Integer.toString(activeUser.getOrder().firstDelivery()
								.getOrderDeliveryID()), "%%DELIVERY_NUMBER%%");
				result = replaceValue(
						result,
						Integer.toString(activeUser.getOrder().firstDelivery()
								.getOrderDeliveryID()), "%%DELIVERY_ID%%");
				// DELIVERY_TOTAL
				result = replaceValue(
						result,
						utils.formatPrice(activeUser.getOrder().firstDelivery()
								.getTotal(), currencyCode, locale),
						"%%DELIVERY_TOTAL%%");
				// SHIPPING
				result = replaceValue(
						result,
						utils.formatPrice(activeUser.getOrder().firstDelivery()
								.getShipping(), currencyCode, locale),
						"%%DELIVERY_SHIPPING%%");
				// TAX
				result = replaceValue(
						result,
						utils.formatPrice(activeUser.getOrder().firstDelivery()
								.getTax(), currencyCode, locale),
						"%%DELIVERY_TAX%%");
				// DELIVERY_SUBTOTAL
				result = replaceValue(
						result,
						utils.formatPrice(activeUser.getOrder().firstDelivery()
								.getSubtotal(), currencyCode, locale),
						"%%DELIVERY_SUBTOTAL%%");
				// DELIVERY_TAXABLESUBTOTAL
				result = replaceValue(
						result,
						utils.formatPrice(activeUser.getOrder().firstDelivery()
								.getTaxableSubtotal(), currencyCode, locale),
						"%%DELIVERY_TAXABLESUBTOTAL%%");
				// DELIVERY_DATE
				result = replaceValue(
						result,
						formatDate(activeUser.getOrder().firstDelivery()
								.getDelivered(), locale), "%%DELIVERY_DATE%%");
				// DELIVERY_LASTMODIFIED
				result = replaceValue(
						result,
						formatDate(activeUser.getOrder().firstDelivery()
								.getLastModified(), locale),
						"%%DELIVERY_LASTMODIFIED%%");
				// DELIVERY_STATUS
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getStatus(), "%%DELIVERY_STATUS%%");
				// DELIVERY_STATUSDETAILS
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getStatusDetails(),
						"%%DELIVERY_STATUSDETAILS%%");
				// DELIVERY_TRACKINGNUMBER
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getTrackingNumber(),
						"%%DELIVERY_TRACKINGNUMBER%%");
				// DELIVERY_WEIGHT
				String value = "";
				if (activeUser.getOrder().firstDelivery().getWeight() != null) {
					value = activeUser.getOrder().firstDelivery().getWeight()
							.toString();
				}
				result = replaceValue(result, value, "%%DELIVERY_WEIGHT%%");
				// DELIVERY_QUANTITY
				value = "";
				if (activeUser.getOrder().firstDelivery().getQuantity() != null) {
					value = activeUser.getOrder().firstDelivery().getQuantity()
							.toString();
				}
				result = replaceValue(result, value, "%%DELIVERY_QUANTITY%%");
				// SHIP_FNAME
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getFirstName(), "%%SHIP_FNAME%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getFirstName(),
						"%%DELIVERY_FIRSTNAME%%");
				// SHIP_LNAME
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getLastName(), "%%SHIP_LNAME%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getLastName(), "%%DELIVERY_LASTNAME%%");
				// SHIP_COMP
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getOrganization(), "%%SHIP_COMP%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getOrganization(), "%%SHIP_COMPANY%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getOrganization(),
						"%%DELIVERY_ORGANIZATION%%");
				// SHIP_ADDR
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getAddress1(), "%%SHIP_ADDR%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getAddress1(), "%%SHIP_ADDRESS%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getAddress1(), "%%DELIVERY_ADDRESS1%%");
				// SHIP_ADDR2
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getAddress2(), "%%SHIP_ADDR2%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getAddress2(), "%%DELIVERY_ADDRESS2%%");
				// SHIP_CITY
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getCity(), "%%SHIP_CITY%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getCity(), "%%DELIVERY_CITY%%");
				// SHIP_STATE
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getState(), "%%SHIP_STATE%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getState(), "%%DELIVERY_STATE%%");
				// SHIP_ZIP
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getPostalCode(), "%%SHIP_ZIP%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getPostalCode(),
						"%%DELIVERY_POSTALCODE%%");
				// SHIP_CNTRY
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getCountry(), "%%SHIP_CNTRY%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getCountry(), "%%DELIVERY_COUNTRY%%");
				// SHIP_PHONE
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getPhone1(), "%%SHIP_PHONE%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getPhone1(), "%%DELIVERY_PHONE1%%");
				// SHIP_PHONE2
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getPhone2(), "%%DELIVERY_PHONE2%%");
				// SHIP_EMAIL
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getEmail1(), "%%SHIP_EMAIL%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getEmail1(), "%%DELIVERY_EMAIL1%%");
				// SHIP_EMAIL2
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getEmail2(), "%%DELIVERY_EMAIL2%%");
				// SHIP_EXTRA
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getExtra1(), "%%SHIP_EXTRA%%");
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getExtra1(), "%%DELIVERY_EXTRA1%%");
				// SHIP_EXTRA2
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getExtra2(), "%%DELIVERY_EXTRA2%%");
				// SHIP_EXTRA3
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getExtra3(), "%%DELIVERY_EXTRA3%%");
				// SHIP_EXTRA4
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getExtra4(), "%%DELIVERY_EXTRA4%%");
				// SHIP_EXTRA5
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getExtra5(), "%%DELIVERY_EXTRA5%%");
				// SHIP_NOTES
				result = replaceValue(result, activeUser.getOrder()
						.firstDelivery().getNotes(), "%%DELIVERY_NOTES%%");
				if (activeUser.getOrder().firstDelivery()
						.getOrderItemCollection() != null
						&& !activeUser.getOrder().firstDelivery()
								.getOrderItemCollection().isEmpty()) {
					result = replaceOrderItems(result, activeUser.getOrder()
							.firstDelivery().getOrderItemCollection(),
							currencyCode, locale, utils);
				}
			}
			if (activeUser != null && activeUser.getCustomer() != null) {
				// CUSTOMER_CUSTOMER_ID
				result = replaceValue(result, Integer.toString(activeUser
						.getCustomer().getCustomerID()),
						"%%CUSTOMER_CUSTOMER_ID%%");
				// CUSTOMER_USER_NAME
				result = replaceValue(result, activeUser.getCustomer()
						.getUserName(), "%%CUSTOMER_USER_NAME%%");
				// CUSTOMER_LASTLOGIN
				result = replaceValue(
						result,
						formatDate(activeUser.getCustomer().getLastLogin(),
								locale), "%%CUSTOMER_LASTLOGIN%%");
				// CUSTOMER_CREATED
				result = replaceValue(
						result,
						formatDate(activeUser.getCustomer().getCreated(),
								locale), "%%CUSTOMER_CREATED%%");
				// CUSTOMER_STATUS
				result = replaceValue(result, activeUser.getCustomer()
						.getStatus(), "%%CUSTOMER_STATUS%%");
				// CUSTOMER_BILL_FIRSTNAME
				result = replaceValue(result, activeUser.getCustomer()
						.getFirstName(), "%%CUSTOMER_BILL_FIRSTNAME%%");
				// CUSTOMER_BILL_LASTNAME
				result = replaceValue(result, activeUser.getCustomer()
						.getLastName(), "%%CUSTOMER_BILL_LASTNAME%%");
				// CUSTOMER_BILL_ORGANIZATION
				result = replaceValue(result, activeUser.getCustomer()
						.getOrganization(), "%%CUSTOMER_BILL_ORGANIZATION%%");
				// CUSTOMER_BILL_ADDR
				result = replaceValue(result, activeUser.getCustomer()
						.getAddress1(), "%%CUSTOMER_BILL_ADDRESS1%%");
				// CUSTOMER_BILL_ADDRESS2
				result = replaceValue(result, activeUser.getCustomer()
						.getAddress2(), "%%CUSTOMER_BILL_ADDRESS2%%");
				// CUSTOMER_BILL_CITY
				result = replaceValue(result, activeUser.getCustomer()
						.getCity(), "%%CUSTOMER_BILL_CITY%%");
				// CUSTOMER_BILL_STATE
				result = replaceValue(result, activeUser.getCustomer()
						.getState(), "%%CUSTOMER_BILL_STATE%%");
				// CUSTOMER_BILL_ZIP
				result = replaceValue(result, activeUser.getCustomer()
						.getPostalCode(), "%%CUSTOMER_BILL_ZIP%%");
				result = replaceValue(result, activeUser.getCustomer()
						.getPostalCode(), "%%CUSTOMER_BILL_POSTALCODE%%");
				// CUSTOMER_BILL_COUNTRY
				result = replaceValue(result, activeUser.getCustomer()
						.getCountry(), "%%CUSTOMER_BILL_COUNTRY%%");
				// CUSTOMER_BILL_PHONE1
				result = replaceValue(result, activeUser.getCustomer()
						.getPhone1(), "%%CUSTOMER_BILL_PHONE1%%");
				// CUSTOMER_BILL_PHONE2
				result = replaceValue(result, activeUser.getCustomer()
						.getPhone2(), "%%CUSTOMER_BILL_PHONE2%%");
				// CUSTOMER_BILL_EMAIL
				result = replaceValue(result, activeUser.getCustomer()
						.getEmail1(), "%%CUSTOMER_BILL_EMAIL1%%");
				// CUSTOMER_BILL_EMAIL2
				result = replaceValue(result, activeUser.getCustomer()
						.getEmail2(), "%%CUSTOMER_BILL_EMAIL2%%");
				// CUSTOMER_BILL_EXTRA
				result = replaceValue(result, activeUser.getCustomer()
						.getExtra1(), "%%CUSTOMER_BILL_EXTRA1%%");
				// CUSTOMER_BILL_EXTRA2
				result = replaceValue(result, activeUser.getCustomer()
						.getExtra2(), "%%CUSTOMER_BILL_EXTRA2%%");
				// CUSTOMER_BILL_EXTRA3
				result = replaceValue(result, activeUser.getCustomer()
						.getExtra3(), "%%CUSTOMER_BILL_EXTRA3%%");
				// CUSTOMER_BILL_EXTRA4
				result = replaceValue(result, activeUser.getCustomer()
						.getExtra4(), "%%CUSTOMER_BILL_EXTRA4%%");
				// CUSTOMER_BILL_EXTRA5
				result = replaceValue(result, activeUser.getCustomer()
						.getExtra5(), "%%CUSTOMER_BILL_EXTRA5%%");
				// CUSTOMER_BILL_NOTES
				result = replaceValue(result, activeUser.getCustomer()
						.getNotes(), "%%CUSTOMER_BILL_NOTES%%");
			}
		} catch (Exception e) {
			log.error(e.toString(), e);
		}
		return result.toString();
	}

	/**
	 * Replaces the given token, or placeholder, with the given value in the
	 * given <code>StringBuffer</code>.
	 * 
	 * @param source
	 *            The <code>StringBuffer</code> being parsed.
	 * @param value
	 *            The value the placeholder is being replaced with.
	 * @param token
	 *            The placeholder being replaced.
	 * @return The parsed <code>StringBuffer</code>.
	 */
	public StringBuffer replaceValue(StringBuffer source, String value,
			String token) {
		if (value == null)
			value = "";
		int start = source.indexOf(token);
		while (start != -1) {
			int end = token.length() + start;
			int endAfterValue = value.length() + start;
			source.replace(start, end, value);
			start = source.indexOf(token, endAfterValue);
		}
		return source;
	}

	/**
	 * Parses a <code>StringBuffer</code>, inserting order item data. Loops
	 * through the "%%START_ITEMS%%" and "%%END_ITEMS%%" placeholders, inserting
	 * data from a given <code>Collection</code> of
	 * {@link com.softslate.commerce.businessobjects.order.OrderItem} objects.
	 * Used to parse the invoice email template upon order completion.
	 * 
	 * @param source
	 *            The <code>StringBuffer</code> being parsed.
	 * @param orderItems
	 *            A <code>Collection</code> of
	 *            {@link com.softslate.commerce.businessobjects.order.OrderItem}
	 *            objects whose data is used to parse the source.
	 * @return The parsed <code>StringBuffer</code>.
	 */
	public StringBuffer replaceOrderItems(StringBuffer source,
			Collection orderItems, String currencyCode, Locale locale,
			BusinessObjectUtils utils) {
		int start = source.indexOf("%%START_ITEMS%%");
		int end = -1;
		if (start != -1) {
			end = source.indexOf("%%END_ITEMS%%");
		}
		if (start != -1 && end != -1) {
			StringBuffer itemsTemplate = new StringBuffer(source.substring(
					start + 15, end));
			source.delete(start, end + 13);
			Iterator i = orderItems.iterator();
			while (i.hasNext()) {
				OrderItem orderItem = (OrderItem) i.next();
				StringBuffer itemBuffer = new StringBuffer(
						itemsTemplate.length());
				itemBuffer.append(itemsTemplate);
				int startField = -1;
				startField = itemBuffer.indexOf("%%CODE%%");
				if (startField != -1) {
					String value = orderItem.getCode();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 8, value);
				}
				startField = itemBuffer.indexOf("%%PRODUCT_ID%%");
				if (startField != -1) {
					itemBuffer.replace(startField, startField + 14,
							Integer.toString(orderItem.getProductID()));
				}
				startField = itemBuffer.indexOf("%%NAME%%");
				if (startField != -1) {
					String value = orderItem.getName();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 8, value);
				}
				startField = itemBuffer.indexOf("%%DESCRIPTION%%");
				if (startField != -1) {
					String value = orderItem.getName();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 15, value);
				}
				startField = itemBuffer.indexOf("%%QUANTITY%%");
				if (startField != -1) {
					itemBuffer.replace(startField, startField + 12,
							Integer.toString(orderItem.getQuantity()));
				}
				startField = itemBuffer.indexOf("%%COST%%");
				if (startField != -1) {
					String value = "";
					if (orderItem.getUnitCost() != null) {
						value = utils.formatPrice(orderItem.getUnitCost(),
								currencyCode, locale);
					}
					itemBuffer.replace(startField, startField + 8, value);
				}
				startField = itemBuffer.indexOf("%%PRICE%%");
				if (startField != -1) {
					String value = "";
					if (orderItem.getUnitPrice() != null) {
						value = utils.formatPrice(orderItem.getUnitPrice(),
								currencyCode, locale);
					}
					itemBuffer.replace(startField, startField + 9, value);
				}
				startField = itemBuffer.indexOf("%%ALT_PRICE%%");
				if (startField != -1) {
					String value = "";
					if (orderItem.getAltPrice() != null) {
						value = utils.formatPrice(orderItem.getAltPrice(),
								currencyCode, locale);
					}
					itemBuffer.replace(startField, startField + 13, value);
				}
				startField = itemBuffer.indexOf("%%WEIGHT%%");
				if (startField != -1) {
					String value = "";
					if (orderItem.getWeight() != null) {
						value = orderItem.getWeight().toString();
					}
					itemBuffer.replace(startField, startField + 10, value);
				}
				startField = itemBuffer.indexOf("%%PRODUCT_WEIGHT%%");
				if (startField != -1) {
					String value = "";
					if (orderItem.getProductWeight() != null) {
						value = orderItem.getProductWeight().toString();
					}
					itemBuffer.replace(startField, startField + 18, value);
				}
				startField = itemBuffer.indexOf("%%TOTAL%%");
				if (startField != -1) {
					String value = "";
					if (orderItem.getTotal() != null) {
						value = utils.formatPrice(orderItem.getTotal(),
								currencyCode, locale);
					}
					itemBuffer.replace(startField, startField + 9, value);
				}
				startField = itemBuffer.indexOf("%%IMAGE%%");
				if (startField != -1) {
					String value = orderItem.getImage();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 9, value);
				}
				startField = itemBuffer.indexOf("%%SHORT_DESCRIPTION%%");
				if (startField != -1) {
					String value = orderItem.getShortDescription();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 21, value);
				}
				startField = itemBuffer.indexOf("%%EXTRA1%%");
				if (startField != -1) {
					String value = orderItem.getExtra1();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 10, value);
				}
				startField = itemBuffer.indexOf("%%EXTRA2%%");
				if (startField != -1) {
					String value = orderItem.getExtra2();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 10, value);
				}
				startField = itemBuffer.indexOf("%%EXTRA3%%");
				if (startField != -1) {
					String value = orderItem.getExtra3();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 10, value);
				}
				startField = itemBuffer.indexOf("%%EXTRA4%%");
				if (startField != -1) {
					String value = orderItem.getExtra4();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 10, value);
				}
				startField = itemBuffer.indexOf("%%EXTRA5%%");
				if (startField != -1) {
					String value = orderItem.getExtra5();
					if (value == null) {
						value = "";
					}
					itemBuffer.replace(startField, startField + 10, value);
				}
				// Parse the item attributes.
				int startAttributes = itemBuffer
						.indexOf("%%START_ATTRIBUTES%%");
				int endAttributes = -1;
				if (startAttributes != -1) {
					endAttributes = itemBuffer.indexOf("%%END_ATTRIBUTES%%");
				}
				if (startAttributes != -1 && endAttributes != -1) {
					StringBuffer attributesTemplate = new StringBuffer(
							itemBuffer.substring(startAttributes + 20,
									endAttributes));
					itemBuffer.delete(startAttributes, endAttributes + 18);

					if (orderItem.getOrderItemAttributes() != null) {
						Iterator i2 = orderItem.getOrderItemAttributes()
								.iterator();
						while (i2.hasNext()) {
							OrderItemAttribute attribute = (OrderItemAttribute) i2
									.next();
							StringBuffer attributeBuffer = new StringBuffer(
									attributesTemplate.length());
							attributeBuffer.append(attributesTemplate);
							int startFieldAttribute = -1;
							startFieldAttribute = attributeBuffer
									.indexOf("%%ATTRIBUTE_CODE%%");
							if (startFieldAttribute != -1) {
								String value = attribute.getAttributeCode();
								if (value == null) {
									value = "";
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 18, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%ATTRIBUTE_NAME%%");
							if (startFieldAttribute != -1) {
								String value = attribute.getAttributeName();
								if (value == null) {
									value = "";
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 18, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%ATTRIBUTE_VALUE%%");
							if (startFieldAttribute != -1) {
								String value = attribute.getAttributeValue();
								if (value == null) {
									value = "";
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 19, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%OIA_WEIGHT%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getWeight() != null) {
									value = attribute.getWeight().toString();
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 10, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%OIA_TOTAL%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getTotal() != null) {
									value = attribute.getTotal().toString();
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 9, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%ATTRIBUTE_COST%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getAttributeUnitCost() != null) {
									value = utils.formatPrice(
											attribute.getAttributeUnitCost(),
											currencyCode, locale);
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 18, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%ATTRIBUTE_PRICE%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getAttributeUnitPrice() != null) {
									value = utils.formatPrice(
											attribute.getAttributeUnitPrice(),
											currencyCode, locale);
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 19, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%ATTRIBUTE_ALT_PRICE%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getAttributeAltPrice() != null) {
									value = utils.formatPrice(
											attribute.getAttributeAltPrice(),
											currencyCode, locale);
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 23, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%ATTRIBUTE_WEIGHT%%");
							if (startFieldAttribute != -1) {
								String value = attribute.getAttributeWeight()
										.toString();
								if (value == null) {
									value = "";
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 20, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%OPTION_CODE%%");
							if (startFieldAttribute != -1) {
								String value = attribute.getOptionCode();
								if (value == null) {
									value = "";
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 15, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%OPTION_NAME%%");
							if (startFieldAttribute != -1) {
								String value = attribute.getOptionName();
								if (value == null) {
									value = "";
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 15, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%OPTION_COST%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getOptionUnitCost() != null) {
									value = utils.formatPrice(
											attribute.getOptionUnitCost(),
											currencyCode, locale);
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 15, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%OPTION_PRICE%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getOptionUnitPrice() != null) {
									value = utils.formatPrice(
											attribute.getOptionUnitPrice(),
											currencyCode, locale);
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 16, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%OPTION_ALT_PRICE%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getOptionAltPrice() != null) {
									value = utils.formatPrice(
											attribute.getOptionAltPrice(),
											currencyCode, locale);
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 20, value);
							}
							startFieldAttribute = attributeBuffer
									.indexOf("%%OPTION_WEIGHT%%");
							if (startFieldAttribute != -1) {
								String value = "";
								if (attribute.getOptionWeight() != null) {
									value = attribute.getOptionWeight()
											.toString();
								}
								attributeBuffer.replace(startFieldAttribute,
										startFieldAttribute + 17, value);
							}
							itemBuffer.insert(startAttributes,
									attributeBuffer.toString());
							startAttributes = startAttributes
									+ attributeBuffer.length();
						}
					}
				}
				source.insert(start, itemBuffer.toString());
				start = start + itemBuffer.length();
			}
		}
		return source;
	}

	public String parseLostPasswordTemplateAdvanced(Customer customer, BaseForm baseForm, String template, String token) {
		StringBuffer result = new StringBuffer(template);
		result = replaceValue(result, customer.getUserName(), "%%USERNAME%%");
		result = replaceValue(result, Integer.toString(customer.getCustomerID()), "%%CUSTOMER_ID%%");
		result = replaceValue(result, (String) baseForm.getSettings().getValue("customerSecureURL")
				+ "/ResetPassword.do?passwordToken=" + token, "%%RESET_PASSWORD_URL%%");
		return parseGenericTemplate(baseForm, result.toString());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.ActionUtils#parseLostPasswordTemplate
	 * (com.softslate.commerce.businessobjects.customer.Customer,
	 * com.softslate.commerce.customer.core.BaseForm, java.lang.String)
	 */
	public String parseLostPasswordTemplate(Customer customer,
			BaseForm baseForm, String template) {
		StringBuffer result = new StringBuffer(template);
		result = replaceValue(result, customer.getDecryptedPassword(),
				"%%PASSWORD%%");
		result = replaceValue(result, customer.getUserName(), "%%USERNAME%%");
		result = replaceValue(result,
				Integer.toString(customer.getCustomerID()), "%%CUSTOMER_ID%%");
		return parseGenericTemplate(baseForm, result.toString());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.ActionUtils#parseLowStockTemplate
	 * (com.softslate.commerce.customer.core.BaseForm, java.util.Map,
	 * java.lang.String)
	 */
	public String parseLowStockTemplate(BaseForm baseForm, Map lowStockEmail,
			String template) {
		StringBuffer result = new StringBuffer(template);
		result = replaceValue(result, (String) lowStockEmail.get("code"),
				"%%PRODUCT_CODE%%");
		result = replaceValue(result, (String) lowStockEmail.get("skuNumber"),
				"%%SKU_NUMBER%%");
		result = replaceValue(result, (String) lowStockEmail.get("skuID"),
				"%%SKU_ID%%");
		result = replaceValue(result, (String) lowStockEmail.get("skuNotes"),
				"%%SKU_NOTES%%");
		result = replaceValue(result, (String) lowStockEmail.get("lowAmount"),
				"%%LOW_STOCK_LEVEL%%");
		result = replaceValue(result,
				(String) lowStockEmail.get("previousLevel"),
				"%%PREVIOUS_LEVEL%%");
		result = replaceValue(result, (String) lowStockEmail.get("newLevel"),
				"%%INVENTORY_LEVEL%%");
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.ActionUtils#sendMultipartEmail(java
	 * .lang.String, java.lang.String, java.lang.String, java.lang.String,
	 * java.lang.String, java.lang.String, java.lang.String, java.lang.String,
	 * java.lang.String, java.lang.String)
	 */
	public void sendMultipartEmail(String toEmails, String ccEmails,
			String bccEmails, String fromEmail, String smtpServer,
			String subject, String textMessage, String htmlMessage,
			String username, String password) throws Exception {
		sendMultipartEmail(toEmails, ccEmails, bccEmails, fromEmail,
				smtpServer, subject, textMessage, htmlMessage, username,
				password, null, null);
	}

	public void sendMultipartEmail(String toEmails, String ccEmails,
			String bccEmails, String fromEmail, String smtpServer,
			String subject, String textMessage, String htmlMessage,
			String username, String password, String smtpPort,
			String characterEncoding) throws Exception {
		sendMultipartEmail(toEmails, ccEmails, bccEmails, fromEmail,
				smtpServer, subject, textMessage, htmlMessage, username,
				password, smtpPort, characterEncoding, null, null, null);
	}

	public void sendMultipartEmail(String toEmails, String ccEmails,
			String bccEmails, String fromEmail, String smtpServer,
			String subject, String textMessage, String htmlMessage,
			String username, String password, String smtpPort,
			String characterEncoding, String attachmentName, byte[] attachment,
			String attachmentType) throws Exception {
		sendMultipartEmail(toEmails, ccEmails, bccEmails, fromEmail,
				smtpServer, subject, textMessage, htmlMessage, username,
				password, smtpPort, characterEncoding, attachmentName,
				attachment, attachmentType, null);
	}

	public void sendMultipartEmail(String toEmails, String ccEmails,
			String bccEmails, String fromEmail, String smtpServer,
			String subject, String textMessage, String htmlMessage,
			String username, String password, String smtpPort,
			String characterEncoding, String attachmentName, byte[] attachment,
			String attachmentType, Map<String, String> extraHeaders)
			throws Exception {

		// Set the host smtp host
		Properties props = new Properties();
		props.put("mail.smtp.host", smtpServer);

		if (smtpPort != null && smtpPort.equals("465")) {
			props.put("mail.smtp.starttls.enable", "true");
			props.put("mail.smtp.socketFactory.port", smtpPort);
			props.put("mail.smtp.socketFactory.class",
					"javax.net.ssl.SSLSocketFactory");
			props.put("mail.smtp.socketFactory.fallback", "false");
		}

		if (username != null && username.length() > 0 && password != null
				&& password.length() > 0) {
			props.put("mail.smtp.auth", "true");
		}

		// Create a mail Session
		Session mailSession = Session.getInstance(props, null);
		mailSession.setDebug(false);

		// Create a message
		Message msg = new MimeMessage(mailSession);

		// Add new headers if necessary
		if (extraHeaders != null) {
			Iterator<Map.Entry<String, String>> i = extraHeaders.entrySet()
					.iterator();
			while (i.hasNext()) {
				Map.Entry<String, String> pairs = (Map.Entry<String, String>) i
						.next();
				msg.addHeader(pairs.getKey(), pairs.getValue());
			}
		}

		// Set the to field
		if (toEmails.length() > 0) {
			String[] toList = toEmails.split(",");
			InternetAddress[] toAddresses = new InternetAddress[toList.length];
			for (int x = 0; x < toList.length; x++) {
				toAddresses[x] = new InternetAddress(toList[x]);
			}
			msg.setRecipients(Message.RecipientType.TO, toAddresses);
		}

		// Set the cc field
		if (ccEmails != null && ccEmails.length() > 0) {
			String[] ccList = ccEmails.split(",");
			InternetAddress[] ccAddresses = new InternetAddress[ccList.length];
			for (int x = 0; x < ccList.length; x++) {
				ccAddresses[x] = new InternetAddress(ccList[x]);
			}
			msg.setRecipients(Message.RecipientType.CC, ccAddresses);
		}

		// Set the bcc field
		if (bccEmails != null && bccEmails.length() > 0) {
			String[] bccList = bccEmails.split(",");
			InternetAddress[] bccAddresses = new InternetAddress[bccList.length];
			for (int x = 0; x < bccList.length; x++) {
				bccAddresses[x] = new InternetAddress(bccList[x]);
			}
			msg.setRecipients(Message.RecipientType.BCC, bccAddresses);
		}

		// Set the from field
		InternetAddress fromAddress = new InternetAddress(fromEmail);
		msg.setFrom(fromAddress);

		// Setting the Subject
		msg.setSubject(subject);

		// Setting the date sent
		msg.setSentDate(new java.util.Date());

		// Create a multi-part to combine the parts
		Multipart multipart = new MimeMultipart("alternative");

		// Create your text message part
		MimeBodyPart messageBodyPart = new MimeBodyPart();
		if (characterEncoding != null && characterEncoding.length() > 0) {
			messageBodyPart.setText(textMessage, characterEncoding);
		} else {
			messageBodyPart.setText(textMessage);
		}

		// Add the text part to the multipart
		multipart.addBodyPart(messageBodyPart);

		// Create the html part
		messageBodyPart = new MimeBodyPart();
		if (characterEncoding != null && characterEncoding.length() > 0) {
			messageBodyPart.setContent(htmlMessage, "text/html; charset=\""
					+ characterEncoding + "\"");
		} else {
			messageBodyPart.setContent(htmlMessage, "text/html");
		}

		// Add html part to multi part
		multipart.addBodyPart(messageBodyPart);

		// Add attachment
		if (StringUtils.isNotBlank(attachmentName)
				&& StringUtils.isNotBlank(attachmentType) && attachment != null
				&& attachment.length > 0) {
			MimeBodyPart att = new MimeBodyPart();
			att.setFileName(attachmentName);
			DataSource src = new ByteArrayDataSource(attachment, attachmentType);
			att.setDataHandler(new DataHandler(src));
			multipart.addBodyPart(att);
		}

		// Associate multi-part with message
		msg.setContent(multipart);

		// Set the smtp port
		int smtpPortInt = 0;
		if (smtpPort != null) {
			try {
				smtpPortInt = Integer.parseInt(smtpPort);
			} catch (Exception e) {
				if (log.isInfoEnabled()) {
					log.info("SMTP port setting cannot be parsed "
							+ "into an integer: ", e);
				}
			}
		}

		// Set the username and password
		if (username != null && username.length() > 0 && password != null
				&& password.length() > 0) {
			Transport tr = mailSession.getTransport("smtp");
			if (smtpPortInt > 0) {
				tr.connect(smtpServer, smtpPortInt, username, password);
			} else {
				tr.connect(smtpServer, username, password);
			}

			// Send the message
			msg.saveChanges();
			tr.sendMessage(msg, msg.getAllRecipients());
			tr.close();
		} else {
			// Send the message without authentication
			Transport.send(msg);
		}

		return;
	}

	/**
	 * Given a <code>String</code> representing a date as stored in the
	 * database, returns a more human-readable version suitable for display.
	 * (Specifically, uses this mask to format the date:
	 * <code>MMM d, yyyy HH:mm:ss</code>.)
	 * 
	 * @param dbDate
	 *            The date as stored in the database.
	 * @param locale
	 *            The local to use to format the date.
	 * @return A more human-readable date suitable for display.
	 */
	public String formatDate(String dbDate, Locale locale) {
		if (dbDate == null) {
			return "";
		}
		SimpleDateFormat formatter = new SimpleDateFormat(
				"yyyy-MM-dd HH:mm:ss:SSS", Locale.getDefault());
		try {
			Date date = formatter.parse(dbDate);
			SimpleDateFormat formatter2 = new SimpleDateFormat(
					"MMM d, yyyy HH:mm:ss", locale);
			return formatter2.format(date);
		} catch (ParseException pe) {
			return dbDate;
		}
	}

	public String findCheckoutScreen(BaseForm baseForm) throws Exception {

		// Delegate to the new CheckoutUtils class
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.findCheckoutScreen(baseForm);
	}

	public boolean validateBillingFields(BaseForm baseForm) throws Exception {

		// Delegate to the new CheckoutUtils class
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.validateBillingFields(baseForm);
	}

	public boolean validateDeliveryFields(BaseForm baseForm) throws Exception {

		// Delegate to the new CheckoutUtils class
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.validateDeliveryFields(baseForm);
	}

	public String makeFilenameNice(String originalFileName) throws Exception {
		String originalExtension = "";
		originalFileName = originalFileName.replaceAll("[^a-zA-Z0-9]", "_");
		int lastUnderscore = originalFileName.lastIndexOf("_");
		if (lastUnderscore != -1) {
			originalExtension = "."
					+ originalFileName.substring(lastUnderscore + 1)
							.toLowerCase();
			originalFileName = originalFileName.substring(0, lastUnderscore);
		}
		if (originalFileName.length() > 99) {
			originalFileName = originalFileName.substring(0, 99);
		}
		return (originalFileName + originalExtension);
	}

	public String readFile(String filename) throws Exception {
		BufferedReader reader = new BufferedReader(new FileReader(filename));
		String line;
		StringBuffer results = new StringBuffer();
		line = reader.readLine();
		while (line != null) {
			results.append(line);
			results.append("\n");
			line = reader.readLine();
		}
		reader.close();
		return results.toString();
	}

	public boolean writeFile(String filename, String fileContents)
			throws Exception {
		BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(
				filename));
		bufferedWriter.write(fileContents);
        bufferedWriter.flush();
		bufferedWriter.close();
		return true;
	}

	public boolean saveFile(String filename, InputStream file) throws Exception {
		OutputStream output = new FileOutputStream(filename);
		byte[] buf = new byte[2048];
		int cnt;
		while ((cnt = file.read(buf)) > 0) {
			output.write(buf, 0, cnt);
		}
		output.close();
		file.close();
		return true;
	}

	public boolean onPreRequestEnd(HttpServletRequest request,
			HttpServletResponse response) {
		// Do nothing by default
		return true;
	}

	public boolean onCustomerPreRequestEnd(HttpServletRequest request,
			HttpServletResponse response) {
		// Do nothing by default
		return true;
	}

	public boolean onAdministratorPreRequestEnd(HttpServletRequest request,
			HttpServletResponse response) {
		// Do nothing by default
		return true;
	}

	public void processAuditLogging(HttpServletRequest request) {
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		try {
			RequestLog rl =  bof
					.createObject(RequestLog.class);
			User user = bof.getUser();
			if (request.getParameter("orderID") != null
					&& request.getParameter("orderID").length() > 0
					&& request.getServletPath() != null
					&& request.getServletPath().startsWith("/administrator")) {
				try {
					rl.setOrderID(new Integer(request.getParameter("orderID")));
				} catch (NumberFormatException nfe) {
					if (log.isWarnEnabled())
						log.warn("Non-numeric value for orderID:  ", nfe);
				}
			} else if (user.getOrder() != null) {
				rl.setOrderID(new Integer(user.getOrder().getOrderID()));
			}
			if (request.getParameter("orderNumber") != null
					&& request.getParameter("orderNumber").length() > 0
					&& request.getServletPath() != null
					&& request.getServletPath().startsWith("/administrator")) {
				try {
					rl.setOrderNumber(new Integer(request
							.getParameter("orderNumber")));
				} catch (NumberFormatException nfe) {
					if (log.isWarnEnabled())
						log.warn("Non-numeric value for orderNumber:  ", nfe);
				}
			} else if (user.getOrder() != null) {
				rl.setOrderNumber(user.getOrder().getOrderNumber());
			}
			if (request.getParameter("customerID") != null
					&& request.getParameter("customerID").length() > 0
					&& request.getServletPath() != null
					&& request.getServletPath().startsWith("/administrator")) {
				try {
					rl.setCustomerID(new Integer(request
							.getParameter("customerID")));
				} catch (NumberFormatException nfe) {
					if (log.isWarnEnabled())
						log.warn("Non-numeric value for customerID:  ", nfe);
				}
			} else if (user.getCustomer() != null) {
				rl.setCustomerID(new Integer(user.getCustomer().getCustomerID()));
				rl.setCustomerUserName(user.getCustomer().getUserName());
			}
			if (user.getAdministrator() != null) {
				rl.setAdministratorID(new Integer(user.getAdministrator()
						.getAdministratorID()));
				rl.setAdministratorUserName(user.getAdministrator()
						.getUserName());
			}
			if (request.getSession() != null) {
				rl.setSessionID(request.getSession().getId());
			}
			rl.setIpAddress(request.getRemoteAddr());
			if (request.getAttribute("Referer") != null) {
				if (((String) request.getAttribute("Referer")).length() > 255) {
					rl.setReferrer(((String) request.getAttribute("Referer"))
							.substring(0, 255));
				} else {
					rl.setReferrer(((String) request.getAttribute("Referer")));
				}
			} else if (request.getHeader("Referer") != null) {
				if (request.getHeader("Referer").length() > 255) {
					rl.setReferrer(request.getHeader("Referer").substring(0,
							255));
				} else {
					rl.setReferrer(request.getHeader("Referer"));
				}
			}
			if (request.getHeader("User-Agent") != null
					&& request.getHeader("User-Agent").length() > 255) {
				rl.setUserAgent(request.getHeader("User-Agent").substring(0,
						255));
			} else {
				rl.setUserAgent(request.getHeader("User-Agent"));
			}
			rl.setServletPath(request.getServletPath());
			if (request.getRequestURL().length() > 255) {
				rl.setRequestURL(request.getRequestURL().substring(0, 255));
			} else {
				rl.setRequestURL(request.getRequestURL().toString());
			}
			rl.setRequestParameters(getRequestParamsDebugMessage(request));
			if (request.getParameter("searchString") != null
					&& request.getParameter("searchString").length() > 0) {
				rl.setSearchString(request.getParameter("searchString"));
			}
			if (request.getServletPath() != null
					&& request.getServletPath().startsWith("/administrator")) {
				rl.setIsAdminRequest(true);
			}

			// Log using log4j, according to its configuration
			if (adminAuditLog.isInfoEnabled() && rl.getIsAdminRequest()) {
				adminAuditLog.info(rl.toString());
			} else if (customerAuditLog.isInfoEnabled()) {
				customerAuditLog.info(rl.toString());
			}

			// Log to the database if called for
			String logRequestsToDatabase = (String) bof.getSettings().getValue(
					"logRequestsToDatabase");
			if (logRequestsToDatabase != null
					&& logRequestsToDatabase.equals("1")) {
				RequestLogProcessor rlp =  bof
						.createObject(RequestLogProcessor.class);
				Map result = rlp.addRequestLog(PropertyUtils.describe(rl));
				if (result != null) {
					request.setAttribute("requestLog", result.get("newRecord"));
				}
			}
		} catch (ApplicationException ae) {
			// Swallow without log
		} catch (Exception e) {
			if (log.isWarnEnabled()) {
				log.warn("Exception processing audit log (swallowing): ", e);
			}
		}
	}

	public void processSavedCartCookie(BaseForm baseForm, Map parameters,
			HttpServletResponse response) {
		String automaticallySaveLastCart = (String) baseForm.getSettings()
				.getValue("automaticallySaveLastCart");
		if (automaticallySaveLastCart != null
				&& automaticallySaveLastCart.equals("1")
				&& (baseForm.getUser().getOrder() == null || baseForm.getUser()
						.getOrder().getOrderID() == 0)) {
			parameters.put("autoSavedCartToken", baseForm.getRequest()
					.getSession().getId());
			String savedCartDays = (String) baseForm.getSettings().getValue(
					"savedCartDays");
			int maxAge = 86400000; // Default to 10000 days
			if (savedCartDays != null && !savedCartDays.equals("0")) {
				try {
					maxAge = Integer.parseInt(savedCartDays) * 86400;
				} catch (NumberFormatException nfe) {
					if (log.isInfoEnabled())
						log.info(nfe);
				}
			}
			Cookie savedCartCookie = new Cookie("sscsctoken", baseForm
					.getRequest().getSession().getId());
			savedCartCookie.setMaxAge(maxAge);
			response.addCookie(savedCartCookie);
		}
	}

	public String processCartAddResults(Map results, BaseForm baseForm) {
		// Add error messages for the missing attributes and skus that were
		// denied for being OOS
		Collection missingAttributes = (Collection) results
				.get("missingAttributes");
		Collection rejectedOrderItems = (Collection) results
				.get("rejectedOrderItems");
		if ((missingAttributes != null && !missingAttributes.isEmpty())
				|| (rejectedOrderItems != null && !rejectedOrderItems.isEmpty())) {
			String firstProductCode = null;
			if (missingAttributes != null && !missingAttributes.isEmpty()) {
				// Add errors for each missing attribute
				if (log.isDebugEnabled())
					log.debug("Adding errors for missing attributes.");
				Iterator i = missingAttributes.iterator();
				while (i.hasNext()) {
					Map missingAttribute = (Map) i.next();
					if (firstProductCode == null) {
						firstProductCode = (String) missingAttribute
								.get("code");
					}
					baseForm.getErrors().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.requiredAttribute",
									(String) missingAttribute
											.get("attributeName"),
									(String) missingAttribute.get("code")));
				}
			}
			if (rejectedOrderItems != null && !rejectedOrderItems.isEmpty()) {
				// Add errors for each OOS sku
				if (log.isDebugEnabled())
					log.debug("Adding errors for OOS items.");
				Iterator j = rejectedOrderItems.iterator();
				while (j.hasNext()) {
					OrderItem oi = (OrderItem) j.next();
					if (firstProductCode == null) {
						firstProductCode = oi.getCode();
					}
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.oosSku", oi.getCode()));
				}
			}
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			// Set the first product code in request - we'll redirect to its
			// product page
			baseForm.getRequest().setAttribute("errorProductCode",
					firstProductCode);
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): "
						+ baseForm.getErrors().toString());
			return "failure";
		}
		// Add non-fatal inventory messages
		addInventoryMessages(results, baseForm);
		return "success";
	}

	public void addInventoryMessages(Map results, BaseForm baseForm) {
		if (log.isDebugEnabled())
			log.debug("Adding inventory messages.");
		// Add message for each adjusted order item
		Collection adjustedOrderItems = (Collection) results
				.get("adjustedOrderItems");
		if (adjustedOrderItems != null && !adjustedOrderItems.isEmpty()) {
			Iterator j = adjustedOrderItems.iterator();
			while (j.hasNext()) {
				OrderItem oi = (OrderItem) j.next();
				baseForm.getMessages()
						.add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("messages.adjustedOrderItem",
										oi.getCode(), Integer.toString(oi
												.getQuantity())));
			}
		}
		// Add message for each newly OOS item
		Collection newOOSSKUs = (Collection) results.get("newOOSSKUs");
		if (newOOSSKUs != null && !newOOSSKUs.isEmpty()) {
			OrderItem previousOI = null;
			Iterator k = newOOSSKUs.iterator();
			while (k.hasNext()) {
				Map sku = (Map) k.next();
				OrderItem oi = (OrderItem) sku.get("orderItem");
				if (previousOI == null || !oi.equals(previousOI)) {
					baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.newOOS", oi.getCode()));
					previousOI = oi;
				}
			}
		}
		saveMessages(baseForm.getRequest(),
				(ActionMessages) baseForm.getMessages());
	}

	public Collection loadEstimatedShippingOptions(BaseForm baseForm) {

		try {
			Collection shippingOptions = null;

			// Check the display settings.
			String displayEstimatedShippingOnCartPage = (String) baseForm
					.getSettings().getValue(
							"displayEstimatedShippingOnCartPage");
			if (displayEstimatedShippingOnCartPage == null
					|| !displayEstimatedShippingOnCartPage.equals("1")) {
				return null;
			}
			if (log.isInfoEnabled())
				log.info("Loading estimated shipping.");

			// Return null if the order is not initialized.
			if (baseForm.getUser().getOrder() == null
					|| baseForm.getUser().getOrder().firstDelivery() == null
					|| baseForm.getUser().getOrder().getOrderShippingRules() == null) {
				if (log.isInfoEnabled())
					log.info("Can't load est. shipping; order not initialized.");
				return null;
			}

			// Set defaults for the state, country and zip if they aren't
			// defined for the order or customer.
			boolean defaultsUsed = false;
			boolean resetCountry = false;
			String customerCountry = null;
			String customerCity = null;
			String customerState = null;
			String customerZIP = null;
			String defaultCountry = (String) baseForm.getSettings().getValue(
					"defaultDeliveryCountry");
			String defaultCity = (String) baseForm.getSettings().getValue(
					"defaultDeliveryCity");
			String defaultState = (String) baseForm.getSettings().getValue(
					"defaultDeliveryState");
			String defaultZIP = (String) baseForm.getSettings().getValue(
					"defaultDeliveryZIP");
			if (baseForm.getUser().getCustomer() != null
					&& baseForm.getUser().getCustomer().getFirstAddress() != null) {
				customerCountry = baseForm.getUser().getCustomer()
						.getFirstAddress().getCountry();
				customerCity = baseForm.getUser().getCustomer()
						.getFirstAddress().getCity();
				customerState = baseForm.getUser().getCustomer()
						.getFirstAddress().getState();
				customerZIP = baseForm.getUser().getCustomer()
						.getFirstAddress().getPostalCode();
			}
			if (baseForm.getUser().getOrder().firstDelivery().getCountry() == null
					|| baseForm.getUser().getOrder().firstDelivery()
							.getCountry().length() == 0) {
				if (customerCountry != null && customerCountry.length() > 0) {
					baseForm.getUser().getOrder().firstDelivery()
							.setCountry(customerCountry);
				} else {
					baseForm.getUser().getOrder().firstDelivery()
							.setCountry(defaultCountry);
					defaultsUsed = true;
				}
				resetCountry = true;
			}
			boolean resetCity = false;
			if (baseForm.getUser().getOrder().firstDelivery().getCity() == null
					|| baseForm.getUser().getOrder().firstDelivery().getCity()
							.length() == 0) {
				if (customerCity != null && customerCity.length() > 0) {
					baseForm.getUser().getOrder().firstDelivery()
							.setCity(customerCity);
				} else {
					baseForm.getUser().getOrder().firstDelivery()
							.setCity(defaultCity);
					defaultsUsed = true;
				}
				resetCity = true;
			}
			boolean resetState = false;
			if (baseForm.getUser().getOrder().firstDelivery().getState() == null
					|| baseForm.getUser().getOrder().firstDelivery().getState()
							.length() == 0) {
				if (customerState != null && customerState.length() > 0) {
					baseForm.getUser().getOrder().firstDelivery()
							.setState(customerState);
				} else {
					baseForm.getUser().getOrder().firstDelivery()
							.setState(defaultState);
					defaultsUsed = true;
				}
				resetState = true;
			}
			boolean resetPostalCode = false;
			if (baseForm.getUser().getOrder().firstDelivery().getPostalCode() == null
					|| baseForm.getUser().getOrder().firstDelivery()
							.getPostalCode().length() == 0) {
				if (customerZIP != null && customerZIP.length() > 0) {
					baseForm.getUser().getOrder().firstDelivery()
							.setPostalCode(customerZIP);
				} else {
					baseForm.getUser().getOrder().firstDelivery()
							.setPostalCode(defaultZIP);
					defaultsUsed = true;
				}
				resetPostalCode = true;
			}

			// If a default is being used but we're supposed to prompt for the
			// information instead of using the defaults, skip the call to load
			// the options.
			String estimatedShippingPromptOnCartPage = (String) baseForm
					.getSettings()
					.getValue("estimatedShippingPromptOnCartPage");
			Boolean estimatedShippingFormValidated = (Boolean) baseForm
					.getRequest().getSession()
					.getAttribute("estimatedShippingFormValidated");
			if (estimatedShippingPromptOnCartPage == null
					|| estimatedShippingPromptOnCartPage.equals("0")
					|| estimatedShippingFormValidated != null || !defaultsUsed) {

				// Load the shipping options for the shipping form
				shippingOptions = baseForm.getBusinessObjectFactory()
						.invokeMultipleCollectionProcessors(
								"activeShippingProcessors",
								"loadShippingOptions",
								PropertyUtils.describe(baseForm));

				// Use shipping rule processor to reduce the possible methods
				// available depending on shipping rules setup by the admin.
				ShippingRuleProcessor shippingRuleProcessor = (ShippingRuleProcessor) baseForm
						.getBusinessObjectFactory().createObject(
								"shippingRuleProcessorImplementer");
				int orderID = baseForm.getUser().getOrder().getOrderID();
				shippingOptions = shippingRuleProcessor
						.findAvailableShippingProcessors(shippingOptions,
								orderID);

				// Use the discount processor to apply shipping discounts to
				// each shipping option.
				CartDiscountProcessor discountProcessor = (CartDiscountProcessor) baseForm
						.getBusinessObjectFactory().createObject(
								"cartDiscountProcessorImplementer");
				shippingOptions = discountProcessor
						.processShippingDiscountsOnShippingOptions(shippingOptions);
				if (log.isInfoEnabled())
					log.info("Estimated shipping options set in session.");
			}

			if (resetCountry) {
				baseForm.getUser().getOrder().firstDelivery().setCountry(null);
			}
			if (resetCity) {
				baseForm.getUser().getOrder().firstDelivery().setCity(null);
			}
			if (resetState) {
				baseForm.getUser().getOrder().firstDelivery().setState(null);
			}
			if (resetPostalCode) {
				baseForm.getUser().getOrder().firstDelivery()
						.setPostalCode(null);
			}

			// Set the options in the session's order object
			if (baseForm.getUser().getOrder() != null) {
				baseForm.getUser().getOrder()
						.setEstimatedShippingOptions(shippingOptions);
			}
			return shippingOptions;
		} catch (Exception e) {
			if (log.isWarnEnabled()) {
				log.warn("Exception loading estimated shipping: ", e);
			}
			return null;
		}

	}

	public Map buildParameterMap(HttpServletRequest request) throws Exception {
		Map paramMap = new HashMap();
		Set keys = request.getParameterMap().keySet();
		Iterator i = keys.iterator();
		while (i.hasNext()) {
			String key = (String) i.next();
			String[] values = (String[]) request.getParameterMap().get(key);
			String value = values[0];
			if (value != null) {
				value = value.trim();
			}
			paramMap.put(key, values[0]);
		}
		return paramMap;
	}

	public String getRequestParamsDebugMessage(HttpServletRequest request) {
		StringBuffer sb = new StringBuffer("Request parameters: ");
		Enumeration e = request.getParameterNames();
		while (e.hasMoreElements()) {
			Object nameOb = e.nextElement();
			if (nameOb instanceof String) {
				String name = (String) nameOb;
				sb.append(name + "=");
				String[] paramValues = request.getParameterValues(name);
				if (name.equals("creditCardNumber") || name.equals("creditCardSecurityCode") || name.equals("password")
						|| name.equals("decryptedPassword") || name.equals("decryptedNumber")
						|| name.equals("payflowProPassword") || name.equals("confirmPassword")
						|| name.equals("apiAuthenticationToken") || name.equals("records")) {
					sb.append("[Not logged for security]");
				} else {
					if (paramValues.length == 1) {
						sb.append(request.getParameter(name));
					} else if (paramValues.length > 1) {
						for (int i = 0; i < paramValues.length; i++) {
							if (i != 0)
								sb.append(", ");
							sb.append(paramValues[i]);
						}
					}
				}
			}
			sb.append(" - ");
		}
		return sb.toString().replace("\\", "\\\\").replace("\n", "\\n")
				.replace("\t", "\\t");
	}

	protected void saveErrors(HttpServletRequest request, ActionMessages errors) {

		// Remove any error messages attribute if none are required
		if ((errors == null) || errors.isEmpty()) {
			request.removeAttribute(Globals.ERROR_KEY);
			return;
		}

		// Save the error messages we need
		request.setAttribute(Globals.ERROR_KEY, errors);
	}

	protected void saveMessages(HttpServletRequest request,
			ActionMessages messages) {

		// Remove any messages attribute if none are required
		if ((messages == null) || messages.isEmpty()) {
			request.removeAttribute(Globals.MESSAGE_KEY);
			return;
		}

		// Save the messages we need
		request.setAttribute(Globals.MESSAGE_KEY, messages);
	}

	public String generateCodeFromName(String name) {
		if (log.isInfoEnabled())
			log.info("Incoming name: " + name);
		String output = name.replaceAll("\\<.*?>", " ");
		Pattern p = Pattern.compile("[^a-zA-Z0-9_.-]+");
		Matcher m = p.matcher(output);
		String replaceWith = "-";
		output = m.replaceAll(replaceWith);
		output = output.toLowerCase();
		if (output.length() > 200) {
			output = output.substring(0, 200);
		}
		if (log.isInfoEnabled())
			log.info("Outgoing code: " + output);
		return output;
	}
	
	public String createTrackingNumberLink(String trackingNumber) {
		String link = null;
		if (trackingNumber == null) {
			return null;
		}
		trackingNumber = trackingNumber.trim();
		trackingNumber = trackingNumber.replaceAll("\\s", "");
		if (trackingNumber.length() == 0) {
			return null;
		}
		if (trackingNumber.startsWith("1Z") || trackingNumber.length() == 11 || trackingNumber.length() == 9) {
			link = "http://wwwapps.ups.com/WebTracking/track?track.x=Track&trackNums="
					+ trackingNumber;
		} else if (trackingNumber.length() == 12
				|| trackingNumber.length() == 15) {
			link = "http://www.fedex.com/Tracking?tracknumbers="
					+ trackingNumber;
		} else if (trackingNumber.length() == 22
				|| trackingNumber.length() == 20
				|| trackingNumber.length() == 13
				|| trackingNumber.length() == 10) {
			link = "https://tools.usps.com/go/TrackConfirmAction.action?tLabels="
					+ trackingNumber;
		}
		return link;
	}

	public void processApplicationMessages(ActionMessages errors, ActionMessages messages,
			Collection<ApplicationMessage> applicationMessages) {
		if (applicationMessages != null) {
			for (ApplicationMessage message : applicationMessages) {
				if (message.getLevel().equals(ApplicationMessage.Level.ERROR)) {
					errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", message.getMessage()));
				} else {
					messages.add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.detail", message.getMessage()));
				}
			}
		}
	}
}
