package com.softslate.commerce.customer.customer;

import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.order.AddressBookAddressForm;

/**
 * 
 * @author Jason McWilliams
 */
public class AddressEditFormAction extends BaseAction {
	static Log log = LogFactory.getLog(AddressEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		AddressBookAddressForm baseForm = (AddressBookAddressForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);
		synchronized (baseForm.getRequest().getSession()) {

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}
			String allowAddressBook = (String) baseForm.getSettings().getValue(
					"allowAddressBook");
			if (!"1".equals(allowAddressBook)) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.addressBookNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Error: " + baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}
			prepareAddressBookForm(baseForm);
			return mapping.findForward("success");
		}
	}

	public void prepareAddressBookForm(AddressBookAddressForm baseForm)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Preparing the account address book form.");
		baseForm.prepareStateAndOtherFields(baseForm.getUser().getCustomer());
		int customerAddressID = 0;
		if (baseForm.getCustomerAddressID() != null) {
			try {
				customerAddressID = Integer.parseInt(baseForm
						.getCustomerAddressID());
			} catch (Exception ex) {
				customerAddressID = 0;
			}
			if (customerAddressID > 0) {
				Collection customerAddresses = baseForm.getUser().getCustomer()
						.getCustomerAddressCollection();
				if (customerAddresses != null) {
					for (CustomerAddress addr : (Collection<CustomerAddress>) customerAddresses) {
						if (addr.getCustomerAddressID() == customerAddressID) {
							BeanUtils.copyProperties(baseForm, addr);
						}
					}
				}
			}
		}
		baseForm.getRequest().setAttribute("addressForm", baseForm);
		return;
	}
}
