package com.softslate.commerce.customer.order;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class used for requests to clear the current cart. Used for
 * requests to "/CartClear.do".
 * <p>
 * The <code>execute</code> method of this class nullifies the
 * <code>order</code> property of the
 * {@link com.softslate.commerce.businessobjects.core.User} instance in the
 * current user's session.
 * 
 * @author David Tobey
 * 
 */
public class CartClearAction extends BaseAction {
	static Log log = LogFactory.getLog(CartClearAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		if (log.isDebugEnabled())
			log.debug("Clearing the cart.");

		synchronized (request.getSession()) {
			if (baseForm.getUser().getOrder() != null) {

				// Nullify the session's customer object
				baseForm.getUser().setOrder(null);

				// Force shipping and taxes to be recalculated
				baseForm.getRequest().getSession().setAttribute(
						"shippingAndTaxesProcessed", "false");

				// Reset the PayPal payer ID
				if (baseForm.getUser().getCustomMap() != null
						&& baseForm.getUser().getCustomMap().get(
								"paypalPayerID") != null) {
					baseForm.getUser().getCustomMap()
							.put("paypalPayerID", null);
				}

				// Delete the savedCartToken cookie so the cart isn't reloaded
				// right away again
				Cookie[] cookies = request.getCookies();
				if (cookies != null) {
					for (int i = 0; i < cookies.length; i++) {
						if (cookies[i] != null && cookies[i].getName() != null
								&& cookies[i].getName().equals("sscsctoken")) {
							Cookie savedCartCookie = cookies[i];
							savedCartCookie.setMaxAge(0);
							response.addCookie(savedCartCookie);
						}
					}
				}

				// Tell the CustomerRequestProcessor to skip loading the saved
				// cart as we forward to another action.
				request.setAttribute("skipLoadSavedCart", "true");

				baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.cartCleared"));
				saveMessages(baseForm.getRequest(), baseForm.getMessages());
			}
			return mapping.findForward("success");
		}
	}
}
