package com.softslate.commerce.customer.payment;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.customer.core.ActionUtils;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.customer.order.CheckoutAction;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import net.authorize.ResponseField;

/**
 * Action class to handle AuthorizeNet Return. This updates the customer session
 * with the data that was recorded in the database during the relay response.
 * 
 * @author Tony Zakula
 */

public class AuthorizeNetDPMReturnAction extends CheckoutAction {
	static Log log = LogFactory.getLog(AuthorizeNetDPMReturnAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting AuthorizeNetDPMReturnAction action.");

		BaseForm baseForm = (BaseForm) form;
		baseForm.initializeProperties(mapping, request);

		synchronized (baseForm.getRequest().getSession()) {

			// Check for a page refresh or session timeout
			if (baseForm.getUser().getOrder() == null) {
				if (log.isInfoEnabled())
					log.info("Session timeout upon returning from DPM");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.sessionTimeout"));
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
				return baseForm.getMapping().findForward(
						"failureSessionTimeout");
			}

			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			Map parameters = actionUtils.buildParameterMap(request);

			// First check what status we received from the relay response
			String ssStatus = (String) parameters.get("ssStatus");
			String responseCode = (String) parameters
					.get(ResponseField.RESPONSE_CODE.getFieldName());
			String responseReasonText = (String) parameters
					.get(ResponseField.RESPONSE_REASON_TEXT.getFieldName());

			if (log.isInfoEnabled())
				log.info("ssStatus = " + ssStatus);

			if (ssStatus == null || ssStatus.length() == 0
					|| ssStatus.equals("failure")) {
				// This should go to the core error page.
				throw new Exception("There was an error in the payment process");
			}

			if (ssStatus.equals("success")) {
				// Everything went fine so check where to redirect them.
				if (log.isDebugEnabled())
					log.debug("responseCode = " + responseCode);
				if (responseCode.equals("1")
						|| (responseCode.equals("3") && responseReasonText
								.indexOf("This transaction has been approved") != -1)) {

					// Approved payment so update their session and show them a
					// receipt
					Map orderParameters = new HashMap();
					orderParameters.put(
							"orderID",
							Integer.toString(baseForm.getUser().getOrder()
									.getOrderID()));
					if (log.isInfoEnabled())
						log.info("orderID = "
								+ baseForm.getUser().getOrder().getOrderID());
					OrderProcessor orderProcessor = (OrderProcessor) baseForm
							.getBusinessObjectFactory().createObject(
									"orderProcessorImplementer");
					Order order = orderProcessor
							.loadOrderFromID(orderParameters);
					baseForm.getUser().setOrder(order);

					// Check that their order is complete in the db - they might
					// have gone back and refreshed after initializing a new
					// cart.
					if (baseForm.getUser().getOrder() == null) {
						if (log.isInfoEnabled())
							log.info("Successful response but could not pick up order");
						baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("messages.sessionTimeout"));
						saveErrors(baseForm.getRequest(),
								(ActionMessages) baseForm.getErrors());
						return baseForm.getMapping().findForward(
								"failureSessionTimeout");
					}

					boolean paymentReceived = false;
					String transId = (String) parameters
							.get(ResponseField.TRANSACTION_ID.getFieldName());
					// Check for a payment
					String completeOrder = (String) baseForm.getSettings()
							.getValue("authorizeNetDPMDelayOrderProcessing");
					// Verify the payment was inserted during checkout.
					Collection payments = baseForm.getUser().getOrder()
							.getPayments();
					if (payments != null && !payments.isEmpty()) {
						Iterator i = payments.iterator();
						// Loop through the order's payments and find one that
						// matches our authorize id just to confirm
						while (i.hasNext()) {
							Payment p = (Payment) i.next();
							if (p.getValue7().equalsIgnoreCase(transId.trim())) {
								paymentReceived = true;
							}
						}
					}
					if (paymentReceived == false) {
						// No payment associated with order
						// They may have picked up a new cart due to
						// something going wrong or had an error
						if (log.isInfoEnabled())
							log.info("No payments associated with order.");
						baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("messages.sessionTimeout"));
						saveErrors(baseForm.getRequest(),
								(ActionMessages) baseForm.getErrors());
						return baseForm.getMapping().findForward(
								"failureSessionTimeout");
					}

					String email = (String) baseForm.getSettings().getValue(
							"authorizeNetDPMEmailCustomerConfirmationWhen");
					// Assume we found a payment so check if we
					// need to complete the order
					if (completeOrder.equals("0")) {
						processOrderComplete(baseForm);
						// if we are completing now, we did not send the emails.
						// Ignore admin setting
						email = "0";
					}

					// Now a few details for the receipt screen
					request.setAttribute("user", baseForm.getUser());
					request.setAttribute("baseForm", baseForm);

					// a value of 1 says we sent the emails on the relay
					// response.
					if (email.equals("1")) {
						return mapping
								.findForward("successOrderCompleteNoEmail");
					}

					// Send to receipt page.
					return mapping.findForward("successOrderComplete");
				}
			} // End check for error on relay response

			// We have an error from Authorize.
			String resultMessage = responseCode + " - " + responseReasonText;
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", resultMessage));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			
			// To avoid Ajax craziness, just forward to the CheckoutPaymentForm is there is a failure and it's set to use one-page checkout.
			String useAjaxOnePageCheckout = baseForm.getSettings().getValue("useAjaxOnePageCheckout");
			if (useAjaxOnePageCheckout != null && useAjaxOnePageCheckout.equals("1")) {
				return baseForm.getMapping().findForward("failurePaymentProcessPaymentForm");
			} else {
				return baseForm.getMapping().findForward("failurePaymentProcess");
			}
		} // End syncro block
	}

}
