package com.softslate.commerce.customer.payment;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.ActionUtils;

/**
 * Struts form class for submissions of payment information for Authorize.net
 * during checkout. This form corresponds to the
 * {@link com.softslate.commerce.businessobjects.payment.AuthorizeNetProcessor}
 * payment processor.
 * <p>
 * The database settings "activePaymentProcessors", "activePaymentForms", and
 * "activePaymentFormTemplate" represent the various payment methods available
 * to users. During checkout, each of the "activePaymentForms" will be
 * instantiated and prepared for display on the checkout payment screen. When
 * the user submits payment information the <code>validate</code> method of
 * the corresponding payment form is invoked.
 * <p>
 * The <code>validate</code> method of this class ensures that the incoming
 * credit card information is valid and complete. It then parses the database
 * settings for various PayflowLink fields and populates the corresponding form
 * properties.
 * 
 * @author David Tobey
 */
public class AuthorizeNetForm extends BasicPaymentForm {

	private static final long serialVersionUID = 2846001494308159452L;

	static Log log = LogFactory.getLog(AuthorizeNetForm.class);

	private String description = null;

	private String customerID = null;

	private String invoiceNumber = null;

	private String poNumber = null;

	private String ipAddress = null;

	private String acceptJsDataValue = null;

	private String acceptJsDataDesc = null;

	public String getCustomerID() {
		return customerID;
	}

	public void setCustomerID(String customerID) {
		this.customerID = customerID;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getInvoiceNumber() {
		return invoiceNumber;
	}

	public void setInvoiceNumber(String invoiceNumber) {
		this.invoiceNumber = invoiceNumber;
	}

	public String getPoNumber() {
		return poNumber;
	}

	public void setPoNumber(String poNumber) {
		this.poNumber = poNumber;
	}

	public String getIpAddress() {
		return ipAddress;
	}

	public void setIpAddress(String ipAddress) {
		this.ipAddress = ipAddress;
	}

	public String getAcceptJsDataValue() {
		return acceptJsDataValue;
	}

	public void setAcceptJsDataValue(String acceptJsDataValue) {
		this.acceptJsDataValue = acceptJsDataValue;
	}

	public String getAcceptJsDataDesc() {
		return acceptJsDataDesc;
	}

	public void setAcceptJsDataDesc(String acceptJsDataDesc) {
		this.acceptJsDataDesc = acceptJsDataDesc;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		validateName();
		if (validateCardAndCode()) {
			validateDate();
		}
		validateAcceptJsParams();
		if (validateCardAndCode()) {
			validateSecurityCode();
		}
		validateType();
		if (validateCardAndCode()) {
			validateNumber();
		}
		validateJcaptcha();
		ActionUtils actionUtils = (ActionUtils) getBusinessObjectFactory()
				.createUtilsObject("actionUtilsImplementer");
		// Populate description, comments, etc., parsing them for variables.
		try {
			setDescription(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetDescription")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setInvoiceNumber(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetInvoiceNum")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setPoNumber(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetPONumber")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setCustomerID(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetCustomerID")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setIpAddress(getRequest().getRemoteAddr());
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		if (log.isDebugEnabled()) {
			log.debug("description: " + getDescription());
			log.debug("invoice: " + getInvoiceNumber());
			log.debug("ponum: " + getPoNumber());
			log.debug("customerID: " + getCustomerID());
			log.debug("ipAddress: " + getIpAddress());
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());

		return getErrors();
	}

	protected void validateAcceptJsParams() {
		if ("1".equals(getSettings().getValue("authorizeNetEnableAcceptJS"))) {
			if (StringUtils.isEmpty(getAcceptJsDataValue()) || StringUtils.isEmpty(getAcceptJsDataDesc())) {
				String message = getMessage("payment", "payment.errorAuthorizeNetAIMProcessorNoNonce", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", message));
			}
		}
	}

	protected boolean validateCardAndCode() {
		return !"1".equals(getSettings().getValue("authorizeNetEnableAcceptJS"));
	}
}
