package com.softslate.commerce.daos.administrator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashSet;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.administrator.Administrator;
import com.softslate.commerce.businessobjects.administrator.Role;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscAdministrator</code> table of the
 * database.
 * 
 * @author David Tobey
 * 
 */
public class AdministratorDAOHibernate extends BaseDAO implements
		AdministratorDAO {

	static Log log = LogFactory.getLog(AdministratorDAOHibernate.class);

	private Administrator administrator = null;

	public Administrator getAdministrator() {
		return (this.administrator);
	}

	public void setAdministrator(Administrator administrator) {
		this.administrator = administrator;
	}

	public void loadAdministratorFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Administrator record = (Administrator) session.get(
					Administrator.class, new Integer(getAdministrator()
							.getAdministratorID()));
			if (record != null) {
				if (record.getPassword() != null) {
					record.setDecryptedPassword(getDAOUtils().getDecrypted(
							record.getPassword(),
							(String) getSettings().getValue(
									"administratorEncryptionType"),
							getAppSettings().getProperty("twoWayKeyFile")));
				}
				PropertyUtils.copyProperties(getAdministrator(), record);
			}
			setAdministrator(record);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public String loadPasswordByID() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"administrator.selectPasswordFromID");
			q.setInteger("administratorID", getAdministrator()
					.getAdministratorID());
			String password = (String) q.uniqueResult();
			return password;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public void loadAdministratorByUserName() throws DataAccessException {
		setAdministrator(loadAdministratorByUserName(getAdministrator().getUserName()));
	}

	public Administrator loadAdministratorByUserName(String administratorUserName) throws DataAccessException {
		Administrator administrator = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "administrator.selectFromUserName");
			q.setString("userName", administratorUserName);
			administrator = (Administrator) q.uniqueResult();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return administrator;
	}

	public void loadFromUserNameAndPassword() throws DataAccessException {
		setAdministrator(loadFromUserNameAndPassword(getAdministrator().getUserName(), getAdministrator().getPassword()));
	}

	public Administrator loadFromUserNameAndPassword(String administratorUserName, String administratorPassword)
			throws DataAccessException {
		Administrator administrator = null;
		if (administratorPassword != null) {
			administratorPassword = getDAOUtils().getEncrypted(administratorPassword,
					(String) getSettings().getValue("administratorEncryptionType"),
					getAppSettings().getProperty("twoWayKeyFile"),
					(String) getSettings().getValue("oneWayEncryptionAlgorithm"));
		}
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "administrator.selectFromUserNameAndPassword");
			q.setString("userName", administratorUserName);
			q.setString("password", administratorPassword);
			administrator = (Administrator) q.uniqueResult();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		if (administrator != null && administrator.getPassword() != null) {
			administrator.setDecryptedPassword(getDAOUtils().getDecrypted(administrator.getPassword(),
					(String) getSettings().getValue("administratorEncryptionType"),
					getAppSettings().getProperty("twoWayKeyFile")));
		}
		return administrator;
	}

	public Administrator loadFromApiUserNameAndPassword(String apiLoginID, String apiAuthenticationToken)
			throws Exception {
		Administrator administrator = null;
		if (apiAuthenticationToken != null) {
			apiAuthenticationToken = getDAOUtils().getEncrypted(apiAuthenticationToken,
					(String) getSettings().getValue("administratorEncryptionType"),
					getAppSettings().getProperty("twoWayKeyFile"),
					(String) getSettings().getValue("oneWayEncryptionAlgorithm"));
		}
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "administrator.selectFromApiUserNameAndPassword");
			q.setString("userName", apiLoginID);
			q.setString("password", apiAuthenticationToken);
			administrator = (Administrator) q.uniqueResult();
			setAdministrator(administrator);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		if (administrator != null && administrator.getPassword() != null) {
			administrator.setDecryptedPassword(getDAOUtils().getDecrypted(administrator.getPassword(),
							(String) getSettings().getValue("administratorEncryptionType"),
							getAppSettings().getProperty("twoWayKeyFile")));
		}
		return administrator;
	}

	public void loadRoles() throws Exception {
		loadRoles(getAdministrator());
	}

	public void loadRoles(Administrator administrator) throws Exception {
		Collection roles = new ArrayList();
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "administrator.loadAdministratorRoles");
			q.setInteger("administratorID", administrator.getAdministratorID());
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString(), he);
		}
		Iterator i = results.iterator();
		while (i.hasNext()) {
			Role role = (Role) i.next();
			roles.add(role.getCode());
		}
		administrator.setRoleCodes(roles);
		return;
	}

	public void insertAdministrator() throws DataAccessException {
		insertAdministrator(false);
	}

	public void insertAdministrator(boolean commit) throws DataAccessException {
		if (getAdministrator().getDecryptedPassword() != null) {
			getAdministrator().setPassword(
					getDAOUtils().getEncrypted(
							getAdministrator().getDecryptedPassword(),
							(String) getSettings().getValue(
									"administratorEncryptionType"),
							getAppSettings().getProperty("twoWayKeyFile"),
							(String) getSettings().getValue("oneWayEncryptionAlgorithm")));
		}
		Session session = getDaoFactory().startSession();
		try {
			session.save(getAdministrator());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateAdministrator() throws DataAccessException {
		updateAdministrator(false);
	}

	public void updateAdministrator(boolean commit) throws DataAccessException {
		// Do not re-encrypt the password if the encryption type is one way.
		String encType = (String) getSettings().getValue(
				"administratorEncryptionType");
		if (encType != null
				&& getAdministrator().getDecryptedPassword() != null
				&& getAdministrator().getDecryptedPassword().length() > 0
				&& !getAdministrator().getDecryptedPassword().equals(
						"[oneWayEncryption]")
				&& !getAdministrator().getDecryptedPassword().equals(
						"[keyPairNoStoreEncryption]")) {
			getAdministrator().setPassword(
					getDAOUtils().getEncrypted(
							getAdministrator().getDecryptedPassword(), encType,
							getAppSettings().getProperty("twoWayKeyFile"),
							(String) getSettings().getValue("oneWayEncryptionAlgorithm")));
		} else {
			// Use the password that's already in the database.
			try {
				String password = loadPasswordByID();
				getAdministrator().setPassword(password);
			} catch (Exception e) {
			}
		}
		Session session = getDaoFactory().startSession();
		try {
			session.update(getAdministrator());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void deleteAdministrator() throws DataAccessException {
		deleteAdministrator(false);
	}

	public void deleteAdministrator(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setAdministrator((Administrator) session.load(Administrator.class,
					new Integer(getAdministrator().getAdministratorID())));
			session.delete(getAdministrator());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public int administratorChangePassword(String currentDecryptedPassword, String newDecryptedPassword,
			int administratorID, String passwordExpires, String previousPasswords) throws DataAccessException {
		int updated = 0;
		String currentPassword = null;
		if (currentDecryptedPassword != null) {
			currentPassword = getDAOUtils().getEncrypted(currentDecryptedPassword,
					(String) getSettings().getValue("administratorEncryptionType"),
					getAppSettings().getProperty("twoWayKeyFile"),
					(String) getSettings().getValue("oneWayEncryptionAlgorithm"));
		}
		String newPassword = null;
		if (newDecryptedPassword != null) {
			newPassword = getDAOUtils().getEncrypted(newDecryptedPassword,
					(String) getSettings().getValue("administratorEncryptionType"),
					getAppSettings().getProperty("twoWayKeyFile"),
					(String) getSettings().getValue("oneWayEncryptionAlgorithm"));
		}
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "administrator.administratorChangePassword");
			q.setString("currentPassword", currentPassword);
			q.setString("newPassword", newPassword);
			q.setString("passwordExpires", passwordExpires);
			q.setString("previousPasswords", previousPasswords);
			q.setInteger("administratorID", administratorID);
			updated = q.executeUpdate();
			getDaoFactory().commitTransaction();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return updated;
	}

}
