create table sscShippingRule (
	shippingRuleID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleBean', 
	code varchar(100) not null unique, 
	name varchar(255), 
	description text, 
	isActive bool not null, 
	ruleType varchar(32), 
	ruleOrder int4 not null, 
	extra1 varchar(255), 
	extra2 varchar(255), 
	extra3 varchar(255), 
	primary key (shippingRuleID)
);

CREATE INDEX IDX_sscShippingRule_name ON sscShippingRule (name);
CREATE INDEX IDX_sscShippingRule_ruleOrder ON sscShippingRule (ruleOrder);

create table sscSKUShippingRule (
	skuShippingRuleID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.shipping.SKUShippingRuleBean', 
	shippingRuleID int4, 
	skuID int4, 
	primary key (skuShippingRuleID)
);

alter table sscSKUShippingRule add constraint FK71A9636769AC6E foreign key (shippingRuleID) references sscShippingRule;
alter table sscSKUShippingRule add constraint FK71A96367E324C735 foreign key (skuID) references sscSKU;


create table sscShippingRuleRange (
	shippingRuleRangeID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleRangeBean', 
	shippingRuleID int4, 
	floor float8 not null, 
	ceiling float8 not null, 
	isActive bool not null, 
	discountType varchar(32), 
	discount float8, 
	pkgDescription text, 
	pkgWeight float8 not null, 
	pkgLength float8 not null, 
	pkgHeight float8 not null, 
	pkgWidth float8 not null, 
	pkgExtra1 varchar(255), 
	pkgExtra2 varchar(255), 
	pkgExtra3 varchar(255), 
	pkgExtra4 varchar(255), 
	pkgExtra5 varchar(255), 
	limitMethods text, 
	limitType varchar(32), 
	primary key (shippingRuleRangeID)
);
alter table sscShippingRuleRange add constraint FK355DB57369AC6E foreign key (shippingRuleID) references sscShippingRule;


create table sscOrderShippingRule (
	orderShippingRuleID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.order.OrderShippingRuleBean', 
	orderID int4, 
	orderItemID int4, 
	total float8, 
	shippingRuleID int4, 
	code varchar(100) not null, 
	name varchar(255), 
	ruleType varchar(32), 
	floor float8, 
	ceiling float8, 
	discountType varchar(32), 
	discount float8, 
	pkgDescription text, 
	pkgWeight float8 not null, 
	pkgLength float8 not null, 
	pkgHeight float8 not null, 
	pkgWidth float8 not null, 
	limitMethods text, 
	limitType varchar(32), 
	extra1 varchar(255), 
	extra2 varchar(255), 
	extra3 varchar(255), 
	pkgExtra1 varchar(255), 
	pkgExtra2 varchar(255), 
	pkgExtra3 varchar(255), 
	pkgExtra4 varchar(255), 
	pkgExtra5 varchar(255), 
	subtotal float8, 
	primary key (orderShippingRuleID)
);
alter table sscOrderShippingRule add constraint FKB5F9F8183AE8A63C foreign key (orderItemID) references sscOrderItem;
alter table sscOrderShippingRule add constraint FKB5F9F818DA8BA36 foreign key (orderID) references sscOrder;

CREATE INDEX IDX_sscOShipRule_orderItemID ON sscOrderShippingRule (orderItemID);
CREATE INDEX IDX_sscOrderShippingRule_total ON sscOrderShippingRule (total);
CREATE INDEX IDX_sscOrderShippingRule_code ON sscOrderShippingRule (code);
CREATE INDEX IDX_sscOrderShippingRule_name ON sscOrderShippingRule (name);
